/* This file is part of Lcrzo.
*/

/*-------------------------------------------------------------*/
lcrzo_bool lcrzo_debug_err_ok=LCRZO_FALSE;

/*-------------------------------------------------------------*/
int lcrzo_debug_err_ok_check(int err,
			     lcrzo_bool alsochecktext)
{ lcrzo_string errmsg;
  lcrzo_bool foundtexterror;
  int err_errno, err_h_errno, err_getlasterror;

  if (!lcrzo_debug_err_ok)
  { return(LCRZO_ERR_OK);
  }

  /* we only check for OK */
  if ( err!=LCRZO_ERR_OK ) 
  { return(LCRZO_ERR_OK);
  }

  /* get system error values and reset those values, otherwise, recursion */
  err_errno=errno;
  err_h_errno=h_errno;
#if defined LCRZODEF_SYSTEM_Unix
  err_getlasterror=0;
#elif defined LCRZODEF_SYSTEM_Windows
  err_getlasterror=GetLastError();
#else
 #error "Unknown value for LCRZODEF_SYSTEM"
#endif

  /* display if there is an error for those integers */
  if ( err_errno || err_h_errno || err_getlasterror )
  { /* reset those values, otherwise, recursion */
    errno=0;
#if defined LCRZODEF_SYSTEM_Unix
    h_errno=0;
#elif defined LCRZODEF_SYSTEM_Windows
    if (err_h_errno) WSASetLastError(0);
    if (err_getlasterror) SetLastError(0);
#else
 #error "Unknown value for LCRZODEF_SYSTEM"
#endif
    /* display */
    printf("err=%d, but subval are not OK:\n", err);
    printf(" errno=%d\n", err_errno);
    printf(" h_errno=%d\n", err_h_errno);
    printf(" getlasterror=%d\n", err_getlasterror);
    printf(" bt: ");
    lcrzo_er(lcrzo_debug_backtrace_fprint(stdout));
    lcrzo_er(lcrzo_stream_flush(stdout));
    lcrzo_er(lcrzo_stream_flush(stderr));
    lcrzo_er(lcrzo_time_sleep_msec(300));
    lcrzo_er(lcrzo_priv_program_exit());
  }

  /* now, check for text */
  if ( !alsochecktext )
  { return(LCRZO_ERR_OK);
  }

  foundtexterror=LCRZO_FALSE;
  lcrzo_er(lcrzo_priv_global_get_errmsglcrzo(&errmsg));
  if (strlen(errmsg))
  { foundtexterror=LCRZO_TRUE;
    printf(" %s\n", errmsg);
  }
  lcrzo_er(lcrzo_string_free2(&errmsg));
  lcrzo_er(lcrzo_priv_global_get_errmsgsys(&errmsg));
  if (strlen(errmsg))
  { foundtexterror=LCRZO_TRUE;
    printf(" %s\n", errmsg);
  }
  lcrzo_er(lcrzo_string_free2(&errmsg));
  if (foundtexterror)
  { printf("err=%d, but subval are not OK:\n", err);
    printf(" bt: ");
    lcrzo_er(lcrzo_debug_backtrace_fprint(stdout));
    lcrzo_er(lcrzo_stream_flush(stdout));
    lcrzo_er(lcrzo_stream_flush(stderr));
    lcrzo_er(lcrzo_time_sleep_msec(300));
    lcrzo_er(lcrzo_priv_program_exit());
  }

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_debug_err_ok_activate(lcrzo_bool yes)
{ lcrzo_debug_err_ok=yes;
  return(LCRZO_ERR_OK);
}

