/* This file is part of Lcrzo.
*/

/*-------------------------------------------------------------*/
int lcrzo_debug_exec_init(lcrzo_conststring command,
			  lcrzo_uint32 *ppid,
			  int *pfd)
{ int pipefd[2];
  int ret, pid;
  
  ret=pipe(pipefd);
  if (ret==-1) return(LCRZO_ERR_FUPIPE);

  pid=fork();
  if (pid==-1)
  { close(pipefd[0]);
    close(pipefd[1]);
    return(LCRZO_ERR_FUFORK);
  }
  if (pid==0)
  { /* child */
    ret=close(pipefd[0]);
    if (ret==-1) return(LCRZO_ERR_FUCLOSE);
    ret=close(STDOUT_FILENO);
    if (ret==-1) return(LCRZO_ERR_FUCLOSE);
    ret=close(STDERR_FILENO);
    if (ret==-1) return(LCRZO_ERR_FUCLOSE);
    ret=dup2(pipefd[1], STDOUT_FILENO);
    if (ret==-1) return(LCRZO_ERR_FUDUP2);
    ret=dup2(pipefd[1], STDERR_FILENO);
    if (ret==-1) return(LCRZO_ERR_FUDUP2);
    ret=execl("/bin/sh", "/bin/sh", "-c", command, NULL);
    if (ret==-1) return(LCRZO_ERR_FUEXECL);
  }
  /* father */
  ret=close(pipefd[1]);
  if (ret==-1) return(LCRZO_ERR_FUCLOSE);
  if (pfd!=NULL) *pfd=pipefd[0];
  if (ppid!=NULL) *ppid=pid;
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_debug_exec_close(lcrzo_uint32 pid,
			   int fd)
{ int ret, status;

  ret=close(fd);
  if (ret==-1) return(LCRZO_ERR_FUCLOSE);
  /* eventually kill the child */
  /*ignore*/kill(pid, SIGTERM);
  /*ignore*/waitpid(pid, &status, 0);
  errno=0;
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_debug_exec_readline(int fd,
			      lcrzo_int32 strmaxsize,
			      lcrzo_string str)
{ lcrzo_int32 strsize;
  char c;
  int ret;
  
  strsize=0;
  while (LCRZO_TRUE)
  { ret=read(fd, &c, 1);
    if (ret==-1)
    { if ( strsize!=0 )
      { /* output the received size */
	str[strsize]='\0';
	return(LCRZO_ERR_OK);
      }
      if (errno==EBADF)
      { /* reached end */
        errno=0;
        return(LCRZO_ERR_OKDATAEND);
      }
      /* fatal error */
      return(LCRZO_ERR_FUREAD);
    }
    if (ret==0)
    { /* no more data */
      if ( strsize!=0 )
      { /* output the received size */
	str[strsize]='\0';
	return(LCRZO_ERR_OK);
      }
      /* reached end */
      return(LCRZO_ERR_OKDATAEND);
    }
    /* terminate at end of line */
    if ( c=='\0' || c=='\n' )
    { str[strsize]='\0';
      return(LCRZO_ERR_OK);
    }
    /* ignore this character */
    if ( c=='\r' )
    { continue;
    }
    /* set buffer */
    if ( strsize<strmaxsize-1 )
    { str[strsize]=c;
      strsize++;
    }
    /* else do not store end */
  }

  return(LCRZO_ERR_OK);
}
