/* This file is part of Lcrzo.
*/

/*-------------------------------------------------------------*/
#if defined LCRZODEF_SYSTEM_Unix
 lcrzo_debug_globalvars lcrzo_debug_globalvars_glo;
#elif defined LCRZODEF_SYSTEM_Windows
 lcrzo_uint32 lcrzo_debug_globalvars_tlsindex=0xFFFFFFFFu;
#else
 #error "Unknown value for LCRZODEF_SYSTEM"
#endif

/*-------------------------------------------------------------*/
int lcrzo_debug_globalvars_init(void)
{
#if defined LCRZODEF_SYSTEM_Unix
#elif defined LCRZODEF_SYSTEM_Windows
  lcrzo_debug_globalvars_tlsindex=TlsAlloc();
  if ((lcrzo_int32)lcrzo_debug_globalvars_tlsindex==-1)
    return(LCRZO_ERR_FUTLSALLOC);
#else
 #error "Unknown value for LCRZODEF_SYSTEM"
#endif

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_debug_globalvars_close(void)
{
#if defined LCRZODEF_SYSTEM_Unix
#elif defined LCRZODEF_SYSTEM_Windows
  { BOOL err;
    err=TlsFree(lcrzo_debug_globalvars_tlsindex);
    if (!err) return(LCRZO_ERR_FUTLSFREE);
    lcrzo_debug_globalvars_tlsindex=0xFFFFFFFFu;
  }
#else
 #error "Unknown value for LCRZODEF_SYSTEM"
#endif

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_debug_globalvars_init_process(void)
{ lcrzo_debug_globalvars *ptr;

#if defined LCRZODEF_SYSTEM_Unix
  ptr=&lcrzo_debug_globalvars_glo;
#elif defined LCRZODEF_SYSTEM_Windows
  { BOOL err;
    ptr=LocalAlloc(LPTR, sizeof(lcrzo_debug_globalvars));
    if (ptr==NULL) return(LCRZO_ERR_FULOCALALLOC);
    err=TlsSetValue(lcrzo_debug_globalvars_tlsindex, ptr);
    if (!err) return(LCRZO_ERR_FUTLSSETVALUE);
  }
#else
 #error "Unknown value for LCRZODEF_SYSTEM"
#endif


#if LCRZO_DEBUG_PROCESSOTHEREXIT_WARN==1
  ptr->processotherexit_warn=LCRZO_TRUE;
#endif

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_debug_globalvars_close_process(void)
{
#if defined LCRZODEF_SYSTEM_Unix
#elif defined LCRZODEF_SYSTEM_Windows
  { BOOL err;
    lcrzo_debug_globalvars *ptr;
    ptr=TlsGetValue(lcrzo_debug_globalvars_tlsindex);
    if (ptr==NULL) return(LCRZO_ERR_OK);
    if (LocalFree(ptr)!=NULL) return(LCRZO_ERR_FULOCALFREE);
    err=TlsSetValue(lcrzo_debug_globalvars_tlsindex, NULL);
    if (!err) return(LCRZO_ERR_FUTLSSETVALUE);
  }
#else
 #error "Unknown value for LCRZODEF_SYSTEM"
#endif

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_debug_globalvars_ptr_get(lcrzo_debug_globalvars **pptr)
{
#if defined LCRZODEF_SYSTEM_Unix
  if (pptr!=NULL)
  { *pptr=&lcrzo_debug_globalvars_glo;
  }
#elif defined LCRZODEF_SYSTEM_Windows
  if (lcrzo_debug_globalvars_tlsindex==0xFFFFFFFFu)
  { if (LCRZODEF_LANG==1)
      puts("lcrzo_debug_globalvars_init() must be called before");
    else
      puts("lcrzo_debug_globalvars_init() doit etre employe d'abord");
    return(LCRZO_ERR_IEINTERNALERROR);
  }
  { LPVOID p;
    p=TlsGetValue(lcrzo_debug_globalvars_tlsindex);
    if (p==NULL) return(LCRZO_ERR_FUTLSGETVALUE); 
    if (pptr!=NULL) *pptr=p;
  }
#endif

  return(LCRZO_ERR_OK);
}
