/* This file is part of Lcrzo.
*/

/*-------------------------------------------------------------*/
#if defined LCRZODEF_SYSTEM_Unix
 #define LCRZO_DEBUG_CAST_FREE 
#elif defined LCRZODEF_SYSTEM_Windows
 /* We should not need it, but system doesn't accept "free((void*)ptr)".
    If you know a better solution, contact me. */
 #define LCRZO_DEBUG_CAST_FREE (void*)
#endif

/*-------------------------------------------------------------*/
#define lcrzo_debug_mem_uint8a_init_uint32(x) ((lcrzo_uint8)(((x)>>24)&0xFF))
#define lcrzo_debug_mem_uint8b_init_uint32(x) ((lcrzo_uint8)(((x)>>16)&0xFF))
#define lcrzo_debug_mem_uint8c_init_uint32(x) ((lcrzo_uint8)(((x)>>8)&0xFF))
#define lcrzo_debug_mem_uint8d_init_uint32(x) ((lcrzo_uint8)((x)&0xFF))
#define lcrzo_debug_mem_uint32_init_uint8x(a,b,c,d) ((lcrzo_uint32)(((lcrzo_uint8)(a<<24))|((lcrzo_uint8)(b<<16))|((lcrzo_uint8)(c<<8))|((lcrzo_uint8)d))
#define lcrzo_debug_mem_data_init_uint32(x,pc) { *(pc++)=lcrzo_debug_mem_uint8a_init_uint32(x); *(pc++)=lcrzo_debug_mem_uint8b_init_uint32(x); *(pc++)=lcrzo_debug_mem_uint8c_init_uint32(x); *(pc++)=lcrzo_debug_mem_uint8d_init_uint32(x); }
#define lcrzo_debug_mem_uint32_init_data(pc,x) { x=(*(pc++))<<24; x|=(*(pc++))<<16; x|=(*(pc++))<<8; x|=(*(pc++)); }


/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/
#define LCRZO_DEBUG_MEMCORRUPT_PATTERN_UNINIT 1
#define LCRZO_DEBUG_MEMCORRUPT_PATTERN_OUTER  2
#define LCRZO_DEBUG_MEMCORRUPT_PATTERN_FREE   3
#define LCRZO_DEBUG_MEMCORRUPT_PATTERN_PROTECTED 4
int lcrzo_debug_memcorrupt_pattern_init(lcrzo_data *ppc, 
					lcrzo_int32 patternsize,
					lcrzo_uint16 patterntype);
int lcrzo_debug_memcorrupt_pattern_init(lcrzo_data *ppc, 
					lcrzo_int32 patternsize,
					lcrzo_uint16 patterntype)
{ lcrzo_uint8 c0, c1, c2; 
  lcrzo_int32 i, j;
  lcrzo_data pc;

  switch (patterntype) { 
  case LCRZO_DEBUG_MEMCORRUPT_PATTERN_UNINIT:
    c0=LCRZO_DEBUG_MEMCORRUPT_UNINITPATTERN_C0;
    c1=LCRZO_DEBUG_MEMCORRUPT_UNINITPATTERN_C1;
    c2=LCRZO_DEBUG_MEMCORRUPT_UNINITPATTERN_C2;
    break;
  case LCRZO_DEBUG_MEMCORRUPT_PATTERN_OUTER:
    c0=LCRZO_DEBUG_MEMCORRUPT_OUTERPATTERN_C0;
    c1=LCRZO_DEBUG_MEMCORRUPT_OUTERPATTERN_C1;
    c2=LCRZO_DEBUG_MEMCORRUPT_OUTERPATTERN_C2;
    break;
  case LCRZO_DEBUG_MEMCORRUPT_PATTERN_FREE:
    c0=LCRZO_DEBUG_MEMCORRUPT_FREEPATTERN_C0;
    c1=LCRZO_DEBUG_MEMCORRUPT_FREEPATTERN_C1;
    c2=LCRZO_DEBUG_MEMCORRUPT_FREEPATTERN_C2;
    break;
  case LCRZO_DEBUG_MEMCORRUPT_PATTERN_PROTECTED:
    c0=0xCC;
    c1=0xCC;
    c2=0xCC;
    break;
  default:
    return(lcrzo_ieuv("lcrzo_debug_memcorrupt_pattern_init",
		      "patterntype"));
  }

  pc=*ppc;
  for ( i=0,j=0 ; i<patternsize ; i++,j++ )
  { if (j==0) *(pc++)=c0;
    else if (j==1) *(pc++)=c1;
    else if (j==2) { *(pc++)=c2; j=-1; }
  }
  *ppc=pc;

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_debug_memcorrupt_pattern_check(lcrzo_conststring patternname,
					 lcrzo_constdata *ppc, 
					 lcrzo_int32 patternsize,
					 lcrzo_uint16 patterntype);
int lcrzo_debug_memcorrupt_pattern_check(lcrzo_conststring patternname,
					 lcrzo_constdata *ppc, 
					 lcrzo_int32 patternsize,
					 lcrzo_uint16 patterntype)
{ lcrzo_uint8 c0, c1, c2, c; 
  lcrzo_int32 i, j;
  lcrzo_constdata pc;

  switch (patterntype) { 
  case LCRZO_DEBUG_MEMCORRUPT_PATTERN_UNINIT:
    c0=LCRZO_DEBUG_MEMCORRUPT_UNINITPATTERN_C0;
    c1=LCRZO_DEBUG_MEMCORRUPT_UNINITPATTERN_C1;
    c2=LCRZO_DEBUG_MEMCORRUPT_UNINITPATTERN_C2;
    break;
  case LCRZO_DEBUG_MEMCORRUPT_PATTERN_OUTER:
    c0=LCRZO_DEBUG_MEMCORRUPT_OUTERPATTERN_C0;
    c1=LCRZO_DEBUG_MEMCORRUPT_OUTERPATTERN_C1;
    c2=LCRZO_DEBUG_MEMCORRUPT_OUTERPATTERN_C2;
    break;
  case LCRZO_DEBUG_MEMCORRUPT_PATTERN_FREE:
    c0=LCRZO_DEBUG_MEMCORRUPT_FREEPATTERN_C0;
    c1=LCRZO_DEBUG_MEMCORRUPT_FREEPATTERN_C1;
    c2=LCRZO_DEBUG_MEMCORRUPT_FREEPATTERN_C2;
    break;
  default:
    return(lcrzo_ieuv("lcrzo_debug_memcorrupt_pattern_check",
		      "patterntype"));
  }

  c=0; /* for the compiler warning */
  pc=*ppc;
  for ( i=0,j=0 ; i<patternsize ; i++,j++ )
  { if (j==0) c=c0;
    else if (j==1) c=c1;
    else if (j==2) { c=c2; j=-1; }
    if ( *pc != c )
    { printf("%s : byte %lu of pattern at %p has changed.\n", 
	     patternname, (lcrzo_uintmax)i, *ppc);
      printf("We found %02X instead of %02X (pattern%02X%02X%02X).\n",
	     *pc, c, c0, c1, c2);
      puts("The bad pattern is :");
      lcrzo_debug_data_print(*ppc, patternsize);
      return(LCRZO_ERR_LOMEMCORRUPT);
    }
    pc++;
  }
  *ppc=pc;

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_debug_memcorrupt_alloclist_add(lcrzo_constdata ptr);
int lcrzo_debug_memcorrupt_alloclist_add(lcrzo_constdata ptr)
{ lcrzo_uintmax alloclistsize;
  lcrzo_debug_globalvars *plpm;

  lcrzo_er(lcrzo_debug_globalvars_ptr_get(&plpm));

  /* add in list */
  alloclistsize=(lcrzo_uintmax)plpm->corrupt_alloclist[0];
  if (alloclistsize==plpm->corrupt_alloclist_maxitems)
  { plpm->corrupt_alloclist_maxitems+=LCRZO_DEBUG_MEMCORRUPT_ALLOCCHUNKS;
    plpm->corrupt_alloclist=(const void **)realloc(
			  LCRZO_DEBUG_CAST_FREE(plpm->corrupt_alloclist),
		          (plpm->corrupt_alloclist_maxitems+1)*sizeof(void*));
    if (plpm->corrupt_alloclist==NULL) return(LCRZO_ERR_FUREALLOC);
  }
  alloclistsize++;
  plpm->corrupt_alloclist[alloclistsize]=(const void*)ptr;
  plpm->corrupt_alloclist[0]=(const void*)alloclistsize;
  /*printf("Set %p\n", ptr);*/
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_debug_memcorrupt_alloclist_del(lcrzo_constdata ptr);
int lcrzo_debug_memcorrupt_alloclist_del(lcrzo_constdata ptr)
{ lcrzo_uintmax alloclistsize, i;
  lcrzo_debug_globalvars *plpm;

  lcrzo_er(lcrzo_debug_globalvars_ptr_get(&plpm));

  alloclistsize=(lcrzo_uintmax)plpm->corrupt_alloclist[0];
  if (alloclistsize==0) return(LCRZO_ERR_LOFREEUNALLOC);
  /*printf("Del %p\n", ptr);*/

  for ( i=1 ; i<=alloclistsize ; i++ )
  { if ( ptr == plpm->corrupt_alloclist[i] )
    { if (alloclistsize!=1)
      { plpm->corrupt_alloclist[i]=plpm->corrupt_alloclist[alloclistsize];
        plpm->corrupt_alloclist[alloclistsize]=NULL;
      }
      alloclistsize--;
      plpm->corrupt_alloclist[0]=(const void *)alloclistsize;
      return(LCRZO_ERR_OK);
    }
  }
  return(LCRZO_ERR_LOFREEUNALLOC);
}


/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/
int lcrzo_debug_memcorrupt_init(void)
{ int i;
  lcrzo_debug_globalvars *plpm;

  lcrzo_er(lcrzo_debug_globalvars_ptr_get(&plpm));

  /* initialize for alloc */
  plpm->corrupt_alloclist_maxitems=LCRZO_DEBUG_MEMCORRUPT_ALLOCCHUNKS;
  plpm->corrupt_alloclist=(const void**)malloc((plpm->corrupt_alloclist_maxitems+1)*sizeof(void*));
  if (plpm->corrupt_alloclist==NULL) return(LCRZO_ERR_FUMALLOC);
  plpm->corrupt_alloclist[0]=0;

  /* initialize for free */
  plpm->corrupt_freelist_nextpost=0;
  for ( i=0 ; i<LCRZO_DEBUG_MEMCORRUPT_FREELIST_SIZE ; i++ )
  { plpm->corrupt_freelist_ptr[i]=NULL;
  }

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_debug_memcorrupt_close(void)
{ lcrzo_data ptr;
  lcrzo_uint32 ptrsize;
  int i;
  lcrzo_debug_globalvars *plpm;

  lcrzo_er(lcrzo_debug_globalvars_ptr_get(&plpm));

  /* close alloc */
  free(LCRZO_DEBUG_CAST_FREE(plpm->corrupt_alloclist));
  plpm->corrupt_alloclist=NULL;
  plpm->corrupt_alloclist_maxitems=0;

  /* close free */
  for ( i=0 ; i<LCRZO_DEBUG_MEMCORRUPT_FREELIST_SIZE ; i++ )
  { ptr=(lcrzo_data)plpm->corrupt_freelist_ptr[i];
    ptrsize=plpm->corrupt_freelist_size[i];
    if (ptr!=NULL)
    { /*printf("Free %p\n", ptr);*/
      #if LCRZO_DEBUG_MEM_CORRUPT==2 || LCRZO_DEBUG_MEM_CORRUPT==3
        lcrzo_er(lcrzo_debug_memprotect_protect(ptr, ptrsize, LCRZO_TRUE));
        lcrzo_er(lcrzo_debug_memprotect_free(ptr, ptrsize));
      #else
        free(ptr);
      #endif
      plpm->corrupt_freelist_ptr[i]=NULL;
    }
  }
  plpm->corrupt_freelist_nextpost=0;

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_debug_memcorrupt_check(lcrzo_constptr ptr)
{ return(lcrzo_debug_memcorrupt_alloc_check(ptr, NULL, NULL, NULL, NULL));
}

/*-------------------------------------------------------------*/
int lcrzo_debug_memcorrupt_check_all(void)
{ lcrzo_uintmax alloclistsize, i, size;
  const void *ptr;
  lcrzo_debug_globalvars *plpm;

  lcrzo_er(lcrzo_debug_globalvars_ptr_get(&plpm));

  /* check allocated data */   
  alloclistsize=(lcrzo_uintmax)plpm->corrupt_alloclist[0];
  for ( i=1 ; i<=alloclistsize ; i++ )
  { ptr=plpm->corrupt_alloclist[i];
    /*printf("CheckA %p\n", ptr);*/
    lcrzo_er(lcrzo_debug_memcorrupt_check((lcrzo_constdata)ptr));
  }

  /* check freed data */
  for ( i=0 ; i<LCRZO_DEBUG_MEMCORRUPT_FREELIST_SIZE ; i++ )
  { ptr=plpm->corrupt_freelist_ptr[i];
    size=plpm->corrupt_freelist_size[i];
    /*printf("CheckF %p %ld\n", ptr, size);*/
    if (ptr!=NULL)
    { lcrzo_er(lcrzo_debug_memcorrupt_free_check((lcrzo_constdata)ptr, size));
    }
  }

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_debug_memcorrupt_check_periodic(void)
{ static int i=1;

  if (LCRZO_DEBUG_MEMCORRUPT_CHECK_FREQUENCY==0) return(LCRZO_ERR_OK);

  i++;
  if ( i>LCRZO_DEBUG_MEMCORRUPT_CHECK_FREQUENCY )
  { i=1;
    return(lcrzo_debug_memcorrupt_check_all());
  }

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_debug_memcorrupt_realloc(lcrzo_int32 newallocsize, 
				   lcrzo_ptr *pptr)
{ lcrzo_ptr ptr;
  lcrzo_uint32 allocsize;

  if (*pptr==NULL)
  { return(lcrzo_debug_memcorrupt_alloc(newallocsize, pptr));
  }

  lcrzo_er(lcrzo_debug_memcorrupt_alloc_check(*pptr, &allocsize,
					      NULL, NULL, NULL));
  lcrzo_er(lcrzo_debug_memcorrupt_alloc(newallocsize, &ptr));
  if (newallocsize<=(lcrzo_int32)allocsize)
  { memcpy(ptr, *pptr, newallocsize);
  }
  else
  { memcpy(ptr, *pptr, allocsize);
    memset((lcrzo_data)ptr+allocsize, 0, newallocsize-allocsize);
  }
  lcrzo_er(lcrzo_debug_memcorrupt_free(pptr));

  if (pptr!=NULL) *pptr=ptr;
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_debug_memcorrupt_add(lcrzo_ptr ptr)
{ lcrzo_er(lcrzo_debug_memcorrupt_alloclist_add(ptr));
  return(LCRZO_ERR_OK);
}


/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/
#if LCRZO_DEBUG_MEM_CORRUPT==1
/* ptroffset 
   pattern
   allocsize smallpattern
   allocsizexor smallpattern
   data
   smallpattern allocsize
   pattern
 */
int lcrzo_debug_memcorrupt_alloc(lcrzo_int32 allocsize, 
				 lcrzo_ptr *pptr)
{ lcrzo_uint32 patternvariablesize, ptroffset;
  lcrzo_uint32 trueallocsize;
  lcrzo_data trueptr, ptr, pc;

  /* compute patternvariablesize */
  #if LCRZO_DEBUG_MEMCORRUPT_OUTERPATTERN_SIZE==0
   patternvariablesize=allocsize+2;
  #else
   patternvariablesize=LCRZO_DEBUG_MEMCORRUPT_OUTERPATTERN_SIZE;
  #endif
  
  /* compute trueallocsize */
  /* trueallocsize=sizeof(lcrzo_uint32)+
     patternvariablesize+
     sizeof(lcrzo_uint32)+
     LCRZO_DEBUG_MEMCORRUPT_SMALLPATTERN_SIZE+
     sizeof(lcrzo_uint32)+
     LCRZO_DEBUG_MEMCORRUPT_SMALLPATTERN_SIZE+
     allocsize+
     LCRZO_DEBUG_MEMCORRUPT_SMALLPATTERN_SIZE+
     sizeof(lcrzo_uint32)+
     patternvariablesize;*/
  ptroffset=sizeof(lcrzo_uint32)+patternvariablesize+
    ((sizeof(lcrzo_uint32)+LCRZO_DEBUG_MEMCORRUPT_SMALLPATTERN_SIZE)<<1);
  trueallocsize=ptroffset+allocsize+sizeof(lcrzo_uint32)+
    LCRZO_DEBUG_MEMCORRUPT_SMALLPATTERN_SIZE+patternvariablesize;

  /* allocate data */
  trueptr=(lcrzo_data)malloc(trueallocsize);
  if (trueptr==NULL) return(LCRZO_ERR_FUMALLOC);

  /* set ptr */
  ptr=trueptr+ptroffset;

  /* now, initialize trueptr */
  pc=trueptr;
  lcrzo_debug_mem_data_init_uint32(ptroffset, pc);
  lcrzo_er(lcrzo_debug_memcorrupt_pattern_init(&pc, patternvariablesize, 
				    LCRZO_DEBUG_MEMCORRUPT_PATTERN_OUTER));
  lcrzo_debug_mem_data_init_uint32(allocsize, pc);
  lcrzo_er(lcrzo_debug_memcorrupt_pattern_init(&pc, 
				    LCRZO_DEBUG_MEMCORRUPT_SMALLPATTERN_SIZE, 
				    LCRZO_DEBUG_MEMCORRUPT_PATTERN_OUTER));
  lcrzo_debug_mem_data_init_uint32(0xFFFFFFFFu^allocsize, pc);
  lcrzo_er(lcrzo_debug_memcorrupt_pattern_init(&pc, 
				    LCRZO_DEBUG_MEMCORRUPT_SMALLPATTERN_SIZE, 
				    LCRZO_DEBUG_MEMCORRUPT_PATTERN_OUTER));
  lcrzo_er(lcrzo_debug_memcorrupt_pattern_init(&pc, 
				    allocsize, 
				    LCRZO_DEBUG_MEMCORRUPT_PATTERN_UNINIT));
  lcrzo_er(lcrzo_debug_memcorrupt_pattern_init(&pc, 
				    LCRZO_DEBUG_MEMCORRUPT_SMALLPATTERN_SIZE, 
				    LCRZO_DEBUG_MEMCORRUPT_PATTERN_OUTER));
  lcrzo_debug_mem_data_init_uint32(allocsize, pc);
  lcrzo_er(lcrzo_debug_memcorrupt_pattern_init(&pc, patternvariablesize, 
				    LCRZO_DEBUG_MEMCORRUPT_PATTERN_OUTER));

  /* display */
  /*lcrzo_debug_data_print(trueptr, trueallocsize);*/

  /* add in list */
  lcrzo_er(lcrzo_debug_memcorrupt_alloclist_add(ptr));

  /* set output */
  *pptr=ptr;
  return(LCRZO_ERR_OK);
}
#endif

/*-------------------------------------------------------------*/
#if LCRZO_DEBUG_MEM_CORRUPT==1
int lcrzo_debug_memcorrupt_free(lcrzo_ptr *pptr)
{ lcrzo_data trueptr, pc, ptr;
  lcrzo_uint32 trueallocsize, ptrsize, nextpost;
  lcrzo_debug_globalvars *plpm;

  lcrzo_er(lcrzo_debug_globalvars_ptr_get(&plpm));

  /* check it was not corrupted */ 
  lcrzo_er(lcrzo_debug_memcorrupt_alloc_check(*pptr, NULL, NULL,
					      (lcrzo_ptr*)&trueptr,
					      &trueallocsize));

  /* remove it from the list */
  lcrzo_er(lcrzo_debug_memcorrupt_alloclist_del(*pptr));

  /* empty data zone */
  pc=trueptr;
  lcrzo_er(lcrzo_debug_memcorrupt_pattern_init(&pc, trueallocsize, 
				    LCRZO_DEBUG_MEMCORRUPT_PATTERN_FREE));

  /* deal with saved freed data */
  nextpost=plpm->corrupt_freelist_nextpost;
  ptr=(lcrzo_data)plpm->corrupt_freelist_ptr[nextpost];
  if (ptr!=NULL)
  { ptrsize=plpm->corrupt_freelist_size[nextpost];
    lcrzo_er(lcrzo_debug_memcorrupt_free_check(ptr, ptrsize));
    /*printf("Free_ %p\n", ptr);*/
    free(ptr);
  }
  plpm->corrupt_freelist_ptr[nextpost]=trueptr;
  plpm->corrupt_freelist_size[nextpost]=trueallocsize;
  nextpost++;
  if (nextpost>=LCRZO_DEBUG_MEMCORRUPT_FREELIST_SIZE) nextpost=0;
  plpm->corrupt_freelist_nextpost=nextpost;

  return(LCRZO_ERR_OK);
}
#endif

/*-------------------------------------------------------------*/
#if LCRZO_DEBUG_MEM_CORRUPT==1
int lcrzo_debug_memcorrupt_alloc_check(lcrzo_constptr ptr,
				       lcrzo_uint32 *pallocsize,
				       lcrzo_uint32 *pptroffset,
				       lcrzo_ptr *ptrueptr,
				       lcrzo_uint32 *ptrueallocsize)
{ lcrzo_uint32 patternvariablesize, offset, readuint32;
  lcrzo_uint32 ptroffset, allocsize;
  lcrzo_constdata pc;

  /* obtain allocsize from bottom data */
  offset=(sizeof(lcrzo_uint32)+
	  LCRZO_DEBUG_MEMCORRUPT_SMALLPATTERN_SIZE)<<1;
  pc=(lcrzo_constdata)ptr-offset;
  lcrzo_debug_mem_uint32_init_data(pc, allocsize);
  lcrzo_er(lcrzo_debug_memcorrupt_pattern_check("Outer_pattern -2", &pc,
				    LCRZO_DEBUG_MEMCORRUPT_SMALLPATTERN_SIZE, 
				    LCRZO_DEBUG_MEMCORRUPT_PATTERN_OUTER));
  lcrzo_debug_mem_uint32_init_data(pc, readuint32);
  lcrzo_er(lcrzo_debug_memcorrupt_pattern_check("Outer_pattern -1", &pc,
				    LCRZO_DEBUG_MEMCORRUPT_SMALLPATTERN_SIZE, 
				    LCRZO_DEBUG_MEMCORRUPT_PATTERN_OUTER));
  if (allocsize!=(readuint32^0xFFFFFFFFu))
  { printf("Allocsize(%08lX) and allocsize(xor(%08lX)=%08lX) are different.\n",
	   (lcrzo_uintmax)allocsize, (lcrzo_uintmax)readuint32, 
	   (lcrzo_uintmax)(readuint32^0xFFFFFFFFu));
    printf("Here is allocsize+outerpattern-2+allocsizexor+outerpattern-1 :\n");
    lcrzo_debug_data_print((lcrzo_constdata)ptr-offset, offset);
    printf("Here is concerned memory (ptr=%p of size %lu) :\n", 
	   ptr, (lcrzo_uintmax)allocsize);
    lcrzo_debug_data_print(ptr, allocsize);
    return(LCRZO_ERR_LOMEMCORRUPT);
  }

  /* check it corresponds with top data */
  pc=(lcrzo_constdata)ptr+allocsize;
  lcrzo_er(lcrzo_debug_memcorrupt_pattern_check("Outer_pattern +1", &pc,
				    LCRZO_DEBUG_MEMCORRUPT_SMALLPATTERN_SIZE, 
				    LCRZO_DEBUG_MEMCORRUPT_PATTERN_OUTER));
  lcrzo_debug_mem_uint32_init_data(pc, readuint32);
  if (allocsize!=readuint32)
  { printf("Bottom allocsize(%08lX) and top allocsize(%08lX) are different.\n",
	   (lcrzo_uintmax)allocsize, (lcrzo_uintmax)readuint32);
    printf("Here is outerpattern+1+allocsize :\n");
    lcrzo_debug_data_print((lcrzo_constdata)ptr+allocsize+
			   LCRZO_DEBUG_MEMCORRUPT_SMALLPATTERN_SIZE, 
			   offset);
    printf("Here is concerned memory (ptr=%p of size %lu) :\n", 
	   ptr, (lcrzo_uintmax)allocsize);
    lcrzo_debug_data_print(ptr, allocsize);
    return(LCRZO_ERR_LOMEMCORRUPT);
  }

  /* now check bottom patternvariablesize */
  #if LCRZO_DEBUG_MEMCORRUPT_OUTERPATTERN_SIZE==0
   patternvariablesize=allocsize+2;
  #else
   patternvariablesize=LCRZO_DEBUG_MEMCORRUPT_OUTERPATTERN_SIZE;
  #endif
  offset=sizeof(lcrzo_uint32)+patternvariablesize+
    ((sizeof(lcrzo_uint32)+LCRZO_DEBUG_MEMCORRUPT_SMALLPATTERN_SIZE)<<1);
  pc=(lcrzo_constdata)ptr-offset;
  lcrzo_debug_mem_uint32_init_data(pc, ptroffset);
  lcrzo_er(lcrzo_debug_memcorrupt_pattern_check("Outer_pattern -3", &pc,
				    patternvariablesize, 
				    LCRZO_DEBUG_MEMCORRUPT_PATTERN_OUTER));
  if (ptroffset!=offset)
  { printf("Ptroffset(%08lX) and computer offset(%08lX) are different.\n", 
	   (lcrzo_uintmax)ptroffset, (lcrzo_uintmax)offset);
    printf("Here is concerned memory (ptr=%p of size %lu) :\n", 
	   ptr, (lcrzo_uintmax)allocsize);
    lcrzo_debug_data_print(ptr, allocsize);
    return(LCRZO_ERR_LOMEMCORRUPT);
  }

  /* now check top patternvariablesize */
  pc=(lcrzo_constdata)ptr+allocsize+sizeof(lcrzo_uint32)+
    LCRZO_DEBUG_MEMCORRUPT_SMALLPATTERN_SIZE;
  lcrzo_er(lcrzo_debug_memcorrupt_pattern_check("Outer_pattern +2", &pc,
				    patternvariablesize, 
				    LCRZO_DEBUG_MEMCORRUPT_PATTERN_OUTER));

  /* set output */
  if (pallocsize!=NULL) *pallocsize=allocsize;
  if (pptroffset!=NULL) *pptroffset=ptroffset;
  if (ptrueptr!=NULL)
    *ptrueptr=(lcrzo_data)((lcrzo_uintmax)((lcrzo_constdata)ptr-ptroffset));
  if (ptrueallocsize!=NULL) *ptrueallocsize=ptroffset+allocsize+
			      sizeof(lcrzo_uint32)+
			      LCRZO_DEBUG_MEMCORRUPT_SMALLPATTERN_SIZE+
			      patternvariablesize;
  return(LCRZO_ERR_OK);
}
#endif

/*-------------------------------------------------------------*/
#if LCRZO_DEBUG_MEM_CORRUPT==1
int lcrzo_debug_memcorrupt_free_check(lcrzo_constptr trueptr,
				      lcrzo_uint32 trueallocsize)
{ lcrzo_constdata pc;
  pc=trueptr;
  lcrzo_er(lcrzo_debug_memcorrupt_pattern_check("Free", &pc, trueallocsize,
				    LCRZO_DEBUG_MEMCORRUPT_PATTERN_FREE));

  return(LCRZO_ERR_OK);
}
#endif

/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/
#if LCRZO_DEBUG_MEM_CORRUPT==2 || LCRZO_DEBUG_MEM_CORRUPT==3
/* bottom_pattern
   protected_memory
   middle_pattern
   bottom_pattern_size
   protected_memory_size
   middle_pattern_size
   top_pattern_size
   allocsize smallpattern
   allocsizexor smallpattern
   data
   protected_memory
   top_pattern
*/
/* bottom_pattern
   protected_memory
   allocsize smallpattern
   allocsizexor smallpattern
   data
   smallpattern
   bottom_pattern_size
   protected_memory_size
   middle_pattern_size
   top_pattern_size
   middle_pattern
   protected_memory
   top_pattern
*/
int lcrzo_debug_memcorrupt_alloc(lcrzo_int32 allocsize, 
				 lcrzo_ptr *pptr)
{ lcrzo_uint32 protectedpages, protectedsize, checkzonesize;
  lcrzo_uint32 middlepages, middlesize, middlepatternsize, middlefixedsize;
  lcrzo_uint32 bottompatternsize, toppatternsize, tmpvar, trueallocsize;
  lcrzo_data trueptr, ptr, pc;

  /* compute protectedsize */
  #if LCRZO_DEBUG_MEMCORRUPT_OUTERPATTERN_SIZE==0
   protectedsize=allocsize+2;
  #else
   protectedsize=LCRZO_DEBUG_MEMCORRUPT_OUTERPATTERN_SIZE;
  #endif
  /* number of memory pages is a multiple of LCRZO_PRIV_PAGESIZE */
  protectedpages=1+(protectedsize-1)/LCRZO_PRIV_PAGESIZE;
  protectedsize=protectedpages*LCRZO_PRIV_PAGESIZE;

  /* middle pages must be aligned on a LCRZO_PRIV_PAGESIZE boundary */
  checkzonesize=4*sizeof(lcrzo_uint32)+
    2*(sizeof(lcrzo_uint32)+LCRZO_DEBUG_MEMCORRUPT_SMALLPATTERN_SIZE); 
#if LCRZO_DEBUG_MEM_CORRUPT==3
  checkzonesize+=LCRZO_DEBUG_MEMCORRUPT_SMALLPATTERN_SIZE;
#endif
  middlefixedsize=allocsize+checkzonesize;
  tmpvar=middlefixedsize%LCRZO_PRIV_PAGESIZE;
  middlepatternsize=(tmpvar==0)?0:LCRZO_PRIV_PAGESIZE-tmpvar;
  middlesize=middlepatternsize+middlefixedsize;
  middlepages=middlesize/LCRZO_PRIV_PAGESIZE;

  /* compute trueallocsize */
  tmpvar=2*protectedpages+middlepages;
#if defined LCRZODEF_SYSTEM_Unix
  trueallocsize=tmpvar*LCRZO_PRIV_PAGESIZE+LCRZO_PRIV_PAGESIZE-1;
#elif defined LCRZODEF_SYSTEM_Windows
  trueallocsize=tmpvar*LCRZO_PRIV_PAGESIZE;
#endif

  /* allocate data */
  lcrzo_er(lcrzo_debug_memprotect_alloc(trueallocsize, (lcrzo_ptr*)&trueptr));

  /* compute bottom pattern size (aligned) */
  tmpvar=LCRZO_PRIV_PAGESIZE-1;
  pc=(lcrzo_data)((lcrzo_uintmax)(trueptr+tmpvar) & (~tmpvar));
  bottompatternsize=pc-trueptr;

  /* compute top pattern size */
  tmpvar=trueallocsize%LCRZO_PRIV_PAGESIZE;
  toppatternsize=tmpvar-bottompatternsize;

  /* set ptr */
#if LCRZO_DEBUG_MEM_CORRUPT==2
  ptr=pc+protectedsize+middlepatternsize+checkzonesize;
#elif LCRZO_DEBUG_MEM_CORRUPT==3
  ptr=pc+protectedsize+
    2*(sizeof(lcrzo_uint32)+LCRZO_DEBUG_MEMCORRUPT_SMALLPATTERN_SIZE);
#endif

  /* now, initialize trueptr */
  pc=trueptr;
  lcrzo_er(lcrzo_debug_memcorrupt_pattern_init(&pc, bottompatternsize, 
					  LCRZO_DEBUG_MEMCORRUPT_PATTERN_OUTER));
  lcrzo_er(lcrzo_debug_memprotect_protect(pc, 
					  protectedpages*LCRZO_PRIV_PAGESIZE, 
					  LCRZO_FALSE));
  pc+=protectedpages*LCRZO_PRIV_PAGESIZE;
#if LCRZO_DEBUG_MEM_CORRUPT==2
  lcrzo_er(lcrzo_debug_memcorrupt_pattern_init(&pc, middlepatternsize, 
				    LCRZO_DEBUG_MEMCORRUPT_PATTERN_OUTER));
  lcrzo_debug_mem_data_init_uint32(bottompatternsize, pc);
  lcrzo_debug_mem_data_init_uint32(protectedsize, pc);
  lcrzo_debug_mem_data_init_uint32(middlepatternsize, pc);
  lcrzo_debug_mem_data_init_uint32(toppatternsize, pc);
  lcrzo_debug_mem_data_init_uint32(allocsize, pc);
  lcrzo_er(lcrzo_debug_memcorrupt_pattern_init(&pc, 
				    LCRZO_DEBUG_MEMCORRUPT_SMALLPATTERN_SIZE, 
				    LCRZO_DEBUG_MEMCORRUPT_PATTERN_OUTER));
  lcrzo_debug_mem_data_init_uint32(0xFFFFFFFFu^allocsize, pc);
  lcrzo_er(lcrzo_debug_memcorrupt_pattern_init(&pc, 
				    LCRZO_DEBUG_MEMCORRUPT_SMALLPATTERN_SIZE, 
				    LCRZO_DEBUG_MEMCORRUPT_PATTERN_OUTER));
  lcrzo_er(lcrzo_debug_memcorrupt_pattern_init(&pc, 
				    allocsize, 
				    LCRZO_DEBUG_MEMCORRUPT_PATTERN_UNINIT));
#elif LCRZO_DEBUG_MEM_CORRUPT==3
  lcrzo_debug_mem_data_init_uint32(allocsize, pc);
  lcrzo_er(lcrzo_debug_memcorrupt_pattern_init(&pc, 
				    LCRZO_DEBUG_MEMCORRUPT_SMALLPATTERN_SIZE, 
				    LCRZO_DEBUG_MEMCORRUPT_PATTERN_OUTER));
  lcrzo_debug_mem_data_init_uint32(0xFFFFFFFFu^allocsize, pc);
  lcrzo_er(lcrzo_debug_memcorrupt_pattern_init(&pc, 
				    LCRZO_DEBUG_MEMCORRUPT_SMALLPATTERN_SIZE, 
				    LCRZO_DEBUG_MEMCORRUPT_PATTERN_OUTER));
  lcrzo_er(lcrzo_debug_memcorrupt_pattern_init(&pc, 
				    allocsize, 
				    LCRZO_DEBUG_MEMCORRUPT_PATTERN_UNINIT));
  lcrzo_er(lcrzo_debug_memcorrupt_pattern_init(&pc, 
				    LCRZO_DEBUG_MEMCORRUPT_SMALLPATTERN_SIZE, 
				    LCRZO_DEBUG_MEMCORRUPT_PATTERN_OUTER));
  lcrzo_debug_mem_data_init_uint32(bottompatternsize, pc);
  lcrzo_debug_mem_data_init_uint32(protectedsize, pc);
  lcrzo_debug_mem_data_init_uint32(middlepatternsize, pc);
  lcrzo_debug_mem_data_init_uint32(toppatternsize, pc);
  lcrzo_er(lcrzo_debug_memcorrupt_pattern_init(&pc, middlepatternsize, 
				    LCRZO_DEBUG_MEMCORRUPT_PATTERN_OUTER));
#endif
  lcrzo_er(lcrzo_debug_memprotect_protect(pc,
					  protectedpages*LCRZO_PRIV_PAGESIZE, 
					  LCRZO_FALSE));
  pc+=protectedpages*LCRZO_PRIV_PAGESIZE;
  lcrzo_er(lcrzo_debug_memcorrupt_pattern_init(&pc, 
				    toppatternsize, 
				    LCRZO_DEBUG_MEMCORRUPT_PATTERN_OUTER));

  /* display */
  /*lcrzo_debug_data_print(trueptr, trueallocsize);*/

  /* add in list */
  lcrzo_er(lcrzo_debug_memcorrupt_alloclist_add(ptr));

  /* set output */
  *pptr=ptr;
  return(LCRZO_ERR_OK);
}
#endif

/*-------------------------------------------------------------*/
#if LCRZO_DEBUG_MEM_CORRUPT==2 || LCRZO_DEBUG_MEM_CORRUPT==3
int lcrzo_debug_memcorrupt_free(lcrzo_ptr *pptr)
{ lcrzo_ptr trueptr, ptr;
  lcrzo_uint32 trueallocsize, ptrsize, nextpost;
  lcrzo_debug_globalvars *plpm;

  lcrzo_er(lcrzo_debug_globalvars_ptr_get(&plpm));

  /* check it was not corrupted */ 
  lcrzo_er(lcrzo_debug_memcorrupt_alloc_check(*pptr, NULL, NULL,
					      &trueptr, &trueallocsize));

  /* remove it from the list */
  lcrzo_er(lcrzo_debug_memcorrupt_alloclist_del(*pptr));

  /* set all pages as protected */
  lcrzo_er(lcrzo_debug_memprotect_protect(trueptr, trueallocsize, LCRZO_FALSE));

  /* deal with saved freed data */
  nextpost=plpm->corrupt_freelist_nextpost;
  ptr=(lcrzo_data)plpm->corrupt_freelist_ptr[nextpost];
  if (ptr!=NULL)
  { ptrsize=plpm->corrupt_freelist_size[nextpost];
    lcrzo_er(lcrzo_debug_memcorrupt_free_check(ptr, ptrsize));
    lcrzo_er(lcrzo_debug_memprotect_protect(ptr, ptrsize, LCRZO_TRUE));
    lcrzo_er(lcrzo_debug_memprotect_free(ptr, ptrsize));
  }
  plpm->corrupt_freelist_ptr[nextpost]=trueptr;
  plpm->corrupt_freelist_size[nextpost]=trueallocsize;
  nextpost++;
  if (nextpost>=LCRZO_DEBUG_MEMCORRUPT_FREELIST_SIZE) nextpost=0;
  plpm->corrupt_freelist_nextpost=nextpost;

  return(LCRZO_ERR_OK);
}
#endif

/*-------------------------------------------------------------*/
#if LCRZO_DEBUG_MEM_CORRUPT==2 || LCRZO_DEBUG_MEM_CORRUPT==3
int lcrzo_debug_memcorrupt_alloc_check(lcrzo_constptr ptr,
				       lcrzo_uint32 *pallocsize,
				       lcrzo_uint32 *pptroffset,
				       lcrzo_ptr *ptrueptr,
				       lcrzo_uint32 *ptrueallocsize)
{ lcrzo_uint32 readuint32, checkzonesize2, checkzonesize4;
  lcrzo_uint32 ptroffset, allocsize, bottompatternsize, trueallocsize;
  lcrzo_uint32 protectedsize, middlepatternsize, toppatternsize;
  lcrzo_constdata pc;

  /* obtain allocsize from bottom data */
  checkzonesize2=2*(sizeof(lcrzo_uint32)+
		    LCRZO_DEBUG_MEMCORRUPT_SMALLPATTERN_SIZE); 
  checkzonesize4=4*sizeof(lcrzo_uint32);
#if LCRZO_DEBUG_MEM_CORRUPT==3
  checkzonesize4+=LCRZO_DEBUG_MEMCORRUPT_SMALLPATTERN_SIZE;
#endif
  pc=(lcrzo_constdata)ptr-checkzonesize2;
  lcrzo_debug_mem_uint32_init_data(pc, allocsize);
  lcrzo_er(lcrzo_debug_memcorrupt_pattern_check("Outer_pattern -2", &pc,
				    LCRZO_DEBUG_MEMCORRUPT_SMALLPATTERN_SIZE, 
				    LCRZO_DEBUG_MEMCORRUPT_PATTERN_OUTER));
  lcrzo_debug_mem_uint32_init_data(pc, readuint32);
  lcrzo_er(lcrzo_debug_memcorrupt_pattern_check("Outer_pattern -1", &pc,
				    LCRZO_DEBUG_MEMCORRUPT_SMALLPATTERN_SIZE, 
				    LCRZO_DEBUG_MEMCORRUPT_PATTERN_OUTER));
  if (allocsize!=(readuint32^0xFFFFFFFFu))
  { printf("Allocsize(%08lX) and allocsize(xor(%08lX)=%08lX) are different.\n",
	   (lcrzo_uintmax)allocsize, (lcrzo_uintmax)readuint32,
	   (lcrzo_uintmax)(readuint32^0xFFFFFFFFu));
    printf("Here is allocsize+outerpattern-2+allocsizexor+outerpattern-1 :\n");
    lcrzo_debug_data_print((lcrzo_constdata)ptr-checkzonesize2,
			   checkzonesize2);
    return(LCRZO_ERR_LOMEMCORRUPT);
  }

  /* obtain other values */
#if LCRZO_DEBUG_MEM_CORRUPT==2
  pc=(lcrzo_constdata)ptr-checkzonesize2-checkzonesize4;
#elif LCRZO_DEBUG_MEM_CORRUPT==3
  pc=(lcrzo_constdata)ptr+allocsize; 
  lcrzo_er(lcrzo_debug_memcorrupt_pattern_check("Outer_pattern +1", &pc,
				    LCRZO_DEBUG_MEMCORRUPT_SMALLPATTERN_SIZE, 
				    LCRZO_DEBUG_MEMCORRUPT_PATTERN_OUTER));
#endif
  lcrzo_debug_mem_uint32_init_data(pc, bottompatternsize);
  lcrzo_debug_mem_uint32_init_data(pc, protectedsize);
  lcrzo_debug_mem_uint32_init_data(pc, middlepatternsize);
  lcrzo_debug_mem_uint32_init_data(pc, toppatternsize);
  /* compute those values */
#if LCRZO_DEBUG_MEM_CORRUPT==2
  ptroffset=bottompatternsize+protectedsize+middlepatternsize+
    checkzonesize4+checkzonesize2;
  trueallocsize=ptroffset+allocsize+protectedsize+toppatternsize;
#elif LCRZO_DEBUG_MEM_CORRUPT==3
  ptroffset=bottompatternsize+protectedsize+checkzonesize2;
  trueallocsize=ptroffset+allocsize+checkzonesize4+middlepatternsize+
    protectedsize+toppatternsize;
#endif

  /* check memory */
  pc=(lcrzo_constdata)ptr-ptroffset;
#if LCRZO_DEBUG_MEM_CORRUPT==2
  lcrzo_er(lcrzo_debug_memcorrupt_pattern_check("Outer_pattern -4", &pc,
					     bottompatternsize, 
				    LCRZO_DEBUG_MEMCORRUPT_PATTERN_OUTER));
  pc+=protectedsize;
  lcrzo_er(lcrzo_debug_memcorrupt_pattern_check("Outer_pattern -3", &pc,
					     middlepatternsize, 
				    LCRZO_DEBUG_MEMCORRUPT_PATTERN_OUTER));
  pc+=checkzonesize4+checkzonesize2+allocsize+protectedsize;
  lcrzo_er(lcrzo_debug_memcorrupt_pattern_check("Outer_pattern +1", &pc,
					     toppatternsize, 
				    LCRZO_DEBUG_MEMCORRUPT_PATTERN_OUTER));
#elif LCRZO_DEBUG_MEM_CORRUPT==3
  lcrzo_er(lcrzo_debug_memcorrupt_pattern_check("Outer_pattern -3", &pc,
					     bottompatternsize, 
				    LCRZO_DEBUG_MEMCORRUPT_PATTERN_OUTER));
  pc+=protectedsize+checkzonesize2+allocsize+checkzonesize4;
  lcrzo_er(lcrzo_debug_memcorrupt_pattern_check("Outer_pattern +2", &pc,
					     middlepatternsize, 
				    LCRZO_DEBUG_MEMCORRUPT_PATTERN_OUTER));
  pc+=protectedsize;
  lcrzo_er(lcrzo_debug_memcorrupt_pattern_check("Outer_pattern +3", &pc,
					     toppatternsize, 
				    LCRZO_DEBUG_MEMCORRUPT_PATTERN_OUTER));
#endif

  /* set output */
  if (pallocsize!=NULL) *pallocsize=allocsize;
  if (pptroffset!=NULL) *pptroffset=ptroffset;
  if (ptrueptr!=NULL)
    *ptrueptr=(lcrzo_data)((lcrzo_uintmax)((lcrzo_constdata)ptr-ptroffset));
  if (ptrueallocsize!=NULL) 
    *ptrueallocsize=trueallocsize;
  return(LCRZO_ERR_OK);
}
#endif

/*-------------------------------------------------------------*/
#if LCRZO_DEBUG_MEM_CORRUPT==2 || LCRZO_DEBUG_MEM_CORRUPT==3
int lcrzo_debug_memcorrupt_free_check(lcrzo_constptr trueptr,
				      lcrzo_uint32 trueallocsize)
{ /* there is no check in this case */
  return(LCRZO_ERR_OK);
  trueptr=trueptr;
  trueallocsize=trueallocsize;
}
#endif
