/* This file is part of Lcrzo.
*/

/*-------------------------------------------------------------*/
int lcrzo_debug_memleak_add(lcrzo_ptr ptr)
{

#if LCRZO_DEBUG_MEM_LEAK==2
  lcrzo_er(lcrzo_debug_memleak_alloc(0, &ptr));
#endif

  return(LCRZO_ERR_OK);
  ptr=ptr; /* just for compiler */
}

/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/
#if LCRZO_DEBUG_MEM_LEAK==1
int lcrzo_debug_memleak_init(void)
{ return(LCRZO_ERR_OK);
}
#endif

/*-------------------------------------------------------------*/
#if LCRZO_DEBUG_MEM_LEAK==1
int lcrzo_debug_memleak_alloc(lcrzo_int32 allocsize, 
			      lcrzo_ptr *pptr)
{ lcrzo_uint32 pid;

  lcrzo_er(lcrzo_debug_pid_get(&pid));
  printf("%09ld mem_alloc : %p %9lu\n", (lcrzo_uintmax)pid, *pptr, 
	 (lcrzo_uintmax)allocsize);
  return(LCRZO_ERR_OK);
}
#endif

/*-------------------------------------------------------------*/
#if LCRZO_DEBUG_MEM_LEAK==1
int lcrzo_debug_memleak_free(lcrzo_ptr *pptr)
{ lcrzo_uint32 pid;

  lcrzo_er(lcrzo_debug_pid_get(&pid));
  printf("%09ld mem_free : %p\n", (lcrzo_uintmax)pid, *pptr);
  return(LCRZO_ERR_OK);
}
#endif

/*-------------------------------------------------------------*/
#if LCRZO_DEBUG_MEM_LEAK==1
int lcrzo_debug_memleak_report(void)
{ return(LCRZO_ERR_OK);
}
#endif

/*-------------------------------------------------------------*/
#if LCRZO_DEBUG_MEM_LEAK==1
int lcrzo_debug_memleak_close(void)
{ return(LCRZO_ERR_OK);
}
#endif

/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/
#if LCRZO_DEBUG_MEM_LEAK==2
int lcrzo_debug_memleak_init(void)
{ lcrzo_debug_globalvars *plpm;
  lcrzo_er(lcrzo_debug_globalvars_ptr_get(&plpm));
  plpm->leak_list=NULL;
  return(LCRZO_ERR_OK);
}
#endif

/*-------------------------------------------------------------*/
#if LCRZO_DEBUG_MEM_LEAK==2
int lcrzo_debug_memleak_list_print_item(void * item[]);
int lcrzo_debug_memleak_list_print_item(void * item[])
{ lcrzo_data ptr;
  lcrzo_intmax ptrsize, btsize, i, pid, displaysize;
  char c;
 
  ptr=(lcrzo_data)item[1]; 
  ptrsize=(lcrzo_intmax)item[2];
  pid=(lcrzo_intmax)item[3];
  printf("%05d %p %9lu\n", (int)pid, ptr, (lcrzo_uintmax)ptrsize);
  btsize=(lcrzo_intmax)item[4];
  if (btsize>LCRZO_DEBUG_BACKTRACE_MAXSIZE) 
  { return(lcrzo_ie("lcrzo_debug_memleak_list_print_item",
		    "btsize>LCRZO_DEBUG_BACKTRACE_MAXSIZE",
		    "btsize>LCRZO_DEBUG_BACKTRACE_MAXSIZE"));
  }
  printf(" ");
  if (btsize!=0)
  { for ( i=5 ; i<btsize+4 ; i++ )
    { printf("0x%lX,", (lcrzo_uintmax)item[i]);
    }
    printf("0x%lX\n", (lcrzo_uintmax)item[btsize+4]);
  }
  else
  { puts("Backtrace is empty (unsupported ?)");
  }
  displaysize=ptrsize;
  if (displaysize>16) displaysize=16;
  for ( i=0 ; i<displaysize ; i++ )
  { printf(" %02X", ptr[i]);
  }
  printf("   ");
  for ( i=0 ; i<displaysize ; i++ )
  { c=ptr[i];
    printf("%c", (char)((c<32||c>126)?'.':c));
  }
  printf("\n");

  return(LCRZO_ERR_OK);
}
int lcrzo_debug_memleak_alloc(lcrzo_int32 allocsize, 
			      lcrzo_ptr *pptr)
{ void * bt[LCRZO_DEBUG_BACKTRACE_MAXSIZE];
  lcrzo_intmax btsize2, allocsize2;
  lcrzo_uintmax pid2;
  lcrzo_int32 btsize, i, j;
  lcrzo_uint32 pid;
  void ** item;
  lcrzo_debug_globalvars *plpm;

  lcrzo_er(lcrzo_debug_globalvars_ptr_get(&plpm));

  /* first, get the back trace */
  lcrzo_er(lcrzo_debug_backtrace_get(LCRZO_DEBUG_BACKTRACE_MAXSIZE,
				    bt, &btsize));
  /*for ( i=0 ; i<btsize-1 ; i++ )
  { printf("%p,", bt[i]);
  }
  printf("%p\n", bt[btsize-1]);*/

  /* allocate list item */
  item=(void**)malloc((5+btsize)*sizeof(void*));
  if (item==NULL) return(LCRZO_ERR_FUMALLOC);

  /* store values in it */
  lcrzo_er(lcrzo_debug_pid_get(&pid));
  item[0]=plpm->leak_list;
  item[1]=(void*)*pptr;
  allocsize2=allocsize;
  item[2]=(void*)allocsize2;
  pid2=pid;
  item[3]=(void*)pid2;
  btsize2=btsize;
  item[4]=(void*)btsize2;
  for ( i=0,j=5 ; i<btsize ; i++,j++ )
  { item[j]=bt[i];
  }

  /* display the item */
  /*printf("ADD %p\n", *pptr);*/
  /*lcrzo_er(lcrzo_debug_memleak_list_print_item(item));*/

  /* insert the item at the beginning of the list */
  plpm->leak_list=item;

  return(LCRZO_ERR_OK);
}
#endif

/*-------------------------------------------------------------*/
#if LCRZO_DEBUG_MEM_LEAK==2
int lcrzo_debug_memleak_free(lcrzo_ptr *pptr)
{ void ** item, **pprec;
  lcrzo_debug_globalvars *plpm;

  lcrzo_er(lcrzo_debug_globalvars_ptr_get(&plpm));
  /*printf("DEL %p\n", *pptr);*/
  item=(void**)plpm->leak_list;
  pprec=NULL;
  while(item!=NULL)
  { /*lcrzo_er(lcrzo_debug_memleak_list_print_item(item));*/
    /* if the address corresponds, remove this item */
    if ( item[1]==*pptr )
    { if (pprec==NULL) plpm->leak_list=item[0];
      else *pprec=item[0];
      free(item);
      return(LCRZO_ERR_OK);
    }
    /* next item */
    pprec=item;
    item=(void**)item[0];
  }
 
  return(LCRZO_ERR_LOFREEUNALLOC);
}
#endif

/*-------------------------------------------------------------*/
#if LCRZO_DEBUG_MEM_LEAK==2
int lcrzo_debug_memleak_report(void)
{ void ** item;
  lcrzo_debug_globalvars *plpm;

  lcrzo_er(lcrzo_debug_globalvars_ptr_get(&plpm));
  item=(void**)plpm->leak_list;
  while(item!=NULL)
  { lcrzo_er(lcrzo_debug_memleak_list_print_item(item));
    /* next item */
    item=(void**)item[0];
  }
  fflush(stdout);
  return(LCRZO_ERR_OK);
}
#endif

/*-------------------------------------------------------------*/
#if LCRZO_DEBUG_MEM_LEAK==2
int lcrzo_debug_memleak_close(void)
{ void ** item, ** itemtofree;
  lcrzo_debug_globalvars *plpm;

  lcrzo_er(lcrzo_debug_globalvars_ptr_get(&plpm));

  item=(void**)plpm->leak_list;
  plpm->leak_list=NULL;
  while(item!=NULL)
  { /*lcrzo_er(lcrzo_debug_memleak_list_print_item(item));*/
    itemtofree=item;
    /* next item */
    item=(void**)item[0];
    /* now free it, we do not need it */
    free(itemtofree);
  }
  return(LCRZO_ERR_OK);
}
#endif
