/* This file is part of Lcrzo.
*/

/*-------------------------------------------------------------*/
#ifdef LCRZODEF_SYSTEM_Unix
 #define LCRZO_PRIV_RESFDOPEN_FD_FILENAME "/dev/null"
#elif defined LCRZODEF_SYSTEM_Windows
 #define LCRZO_PRIV_RESFDOPEN_FD_FILENAME "c:\\winzip.log"
#else
 #error "Unknown value for LCRZODEF_SYSTEM"
#endif

/*-------------------------------------------------------------*/
int lcrzo_debug_resfdopen_check_err(lcrzo_uint32 pid,
				    int fd,
                                    int previousfd);
int lcrzo_debug_resfdopen_check_err(lcrzo_uint32 pid,
				    int fd,
                                    int previousfd)
{
  if (fd!=previousfd+1)
  { if (fd==previousfd+2)
    { printf("Warning[%lu] : fd %d is open\n",
	     (lcrzo_uintmax)pid, previousfd+1);
    }
    else
    { printf("Warning[%lu] : fd %d-%d are open\n", 
	     (lcrzo_uintmax)pid, previousfd+1, fd-1);
    }
  }
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_debug_resfdopen_check2(void);
int lcrzo_debug_resfdopen_check2(void)
{ int ret, fd, previousfd, themax;
  char *popened;
  lcrzo_uint32 pid;

#if defined LCRZODEF_SYSTEM_Linux || defined LCRZODEF_SYSTEM_Solaris || defined LCRZODEF_SYSTEM_OpenBSD
  { struct rlimit lim;
    ret=getrlimit(RLIMIT_NOFILE, &lim);
    if (ret==-1)
    { return(LCRZO_ERR_FUGETRLIMIT);
    }
    themax=lim.rlim_cur;
  }
#elif defined LCRZODEF_SYSTEM_FreeBSD
  /* we should use getrlimit but the returned value cannot really be reached */
  themax=OPEN_MAX;
#elif defined LCRZODEF_SYSTEM_Windows
  themax=2048;
#else
 #error "Unknown value for LCRZODEF_SYSTEM"
#endif

  lcrzo_er(lcrzo_debug_pid_get(&pid));

  /* alloc to decide what has to be closed */
  popened=(char*)malloc(themax);
  if (popened==NULL) return(LCRZO_ERR_FUMALLOC);
  memset(popened, 0, themax);

  /* check */
  for ( fd=-1,previousfd=2 ; fd<themax-1 ; )
  { fd=LAURENT_open(LCRZO_PRIV_RESFDOPEN_FD_FILENAME, O_RDONLY);
    /* last reached */
    if (fd==-1)
    { if ( errno==ENOENT && previousfd==2 )
      { printf("debug_resfdopen: error : file %s does not exist\n",
	       LCRZO_PRIV_RESFDOPEN_FD_FILENAME);
        free(popened);
        return(LCRZO_ERR_IEINTERNALERROR);
      }
      if ( errno!=EMFILE && errno!=ENFILE )
      { printf("Warning[%lu] : errno=%d after fd=%d\n",
	       (lcrzo_uintmax)pid, errno, previousfd);
      }
      ret=lcrzo_debug_resfdopen_check_err(pid, themax, previousfd);
      if (ret!=LCRZO_ERR_OK) { free(popened); return(ret); }
      break;
    }
    /* check if skiped a file descriptor */
    ret=lcrzo_debug_resfdopen_check_err(pid, fd, previousfd);
    if (ret!=LCRZO_ERR_OK) { free(popened); return(ret); }
    /* set if we open it */
    popened[fd]=1;
    previousfd=fd;
  }

  /* check if the max is the good value */
#if defined LCRZODEF_SYSTEM_Linux || defined LCRZODEF_SYSTEM_Solaris || defined LCRZODEF_SYSTEM_OpenBSD || defined LCRZODEF_SYSTEM_Windows
  fd=LAURENT_open(LCRZO_PRIV_RESFDOPEN_FD_FILENAME, O_RDONLY);
  if (fd!=-1)
  { printf("Warning[%lu] : the maximum fd could be %d instead of %d\n",
	   (lcrzo_uintmax)pid, fd+1, themax);
    ret=LAURENT_close(fd);
    if (ret==-1)
    { printf("Warning[%lu] : error in closing fd %d\n",
	     (lcrzo_uintmax)pid, fd);
    }
  }
#elif defined LCRZODEF_SYSTEM_FreeBSD
  /* we used a fake themax, so we cannot check */
#else
 #error "Unknown value for LCRZODEF_SYSTEM"
#endif

  /* close them all */
  for ( fd=0 ; fd<themax ; fd++ )
  { if (popened[fd])
    { ret=LAURENT_close(fd);
      if (ret==-1)
      { printf("Warning[%lu] : error in closing fd %d\n",
	       (lcrzo_uintmax)pid, fd);
      }
    }
  }
  free(popened);

  /* not to generate an "err==OK, but errno!=0" message */
  errno=0;
#if defined LCRZODEF_SYSTEM_Unix
#elif defined LCRZODEF_SYSTEM_Windows
  SetLastError(0);
#else
 #error "Unknown value for LCRZODEF_SYSTEM"
#endif
  
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_debug_resfdopen_check(void)
#if defined LCRZODEF_SYSTEM_Unix
{ return(lcrzo_debug_resfdopen_check2());
}
#elif defined LCRZODEF_SYSTEM_Windows
{ int ret, ret2;

  lcrzo_er(lcrzo_debug_cs_resfdopen_enter());
  ret=lcrzo_debug_resfdopen_check2();
  ret2=lcrzo_debug_cs_resfdopen_leave();
  if (ret!=LCRZO_ERR_OK) return(ret); 
  if (ret2!=LCRZO_ERR_OK) return(ret2); 

  return(LCRZO_ERR_OK);
}
#else
 #error "Unknown value for LCRZODEF_SYSTEM"
#endif
