/* This file is part of Lcrzo.
*/

/* This file is part of Lcrzo.
*/

#ifdef LCRZODEF_SYSTEM_Unix
  /* this function is not needed*/

#elif defined LCRZODEF_SYSTEM_Windows

/*-------------------------------------------------------------*/
int lcrzo_priv_conf_devicenumbername_item_cmp(lcrzo_constptr pelem,
					      lcrzo_ptr pinfos,
					      lcrzo_bool *pyes);
int lcrzo_priv_conf_devicenumbername_item_cmp(lcrzo_constptr pelem,
					      lcrzo_ptr pinfos,
					      lcrzo_bool *pyes)
{ const lcrzo_priv_conf_devicenumbername d1=*(const lcrzo_priv_conf_devicenumbername *)pelem;
  lcrzo_priv_conf_devicenumbername d2=*(lcrzo_priv_conf_devicenumbername *)pinfos;
  lcrzo_bool yes;
  yes=LCRZO_FALSE;

  if ( d1.number==d2.number ) yes=LCRZO_TRUE;
  if (pyes!=NULL) *pyes=yes;
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_conf_devicenumbername_item_add(lcrzo_uint32 devicenumber,
                                         lcrzo_device devicename)
{ lcrzo_priv_conf_devicenumbername dnn;
  lcrzo_priv_confvars *ptrcv;
  lcrzo_int32 pos;
  int ret;
  
  lcrzo_er(lcrzo_priv_confvars_ptr_get(&ptrcv));

  /* set structure */
  dnn.number=devicenumber;
  lcrzo_er(lcrzo_device_init_noalias(devicename, dnn.name));
  
  /* search if devicenumber is already in the list */
  ret=lcrzo_list_search_all2(&(ptrcv->list_devicenumbername),
			     &lcrzo_priv_conf_devicenumbername_item_cmp, 
			     &dnn, &pos, NULL);
  if ( ret!=LCRZO_ERR_OKSEARCHNOTFOUND && ret!=LCRZO_ERR_OK )
    return(ret);
  if ( ret==LCRZO_ERR_OK )
  { /* we found it, so we replace it */
    lcrzo_er(lcrzo_list_replace_pos(&(ptrcv->list_devicenumbername),
				    pos, &dnn));
  }
  else
  { /* we add it to the list */
    lcrzo_er(lcrzo_list_add_last(&(ptrcv->list_devicenumbername), &dnn));
  }

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_conf_devicenumbername_item_get(lcrzo_uint32 devicenumber,
                                         lcrzo_device devicename);
int lcrzo_priv_conf_devicenumbername_item_get(lcrzo_uint32 devicenumber,
                                         lcrzo_device devicename)
{ lcrzo_priv_conf_devicenumbername dnn, dnn2;
  lcrzo_priv_confvars *ptrcv;
  lcrzo_int32 pos;
  int ret;
  
  lcrzo_er(lcrzo_priv_confvars_ptr_get(&ptrcv));

  /* set structure */
  dnn.number=devicenumber;
  lcrzo_er(lcrzo_device_initdefault(dnn.name));
  
  /* search if devicenumber is in the list */
  ret=lcrzo_list_search_all2(&(ptrcv->list_devicenumbername),
			     &lcrzo_priv_conf_devicenumbername_item_cmp, 
			     &dnn, &pos, &dnn2);
  if ( ret==LCRZO_ERR_OK )
  { /* we found it */
    lcrzo_er(lcrzo_device_init(dnn2.name, devicename));
  }

  return(ret);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_conf_devicenumbername_debug_fprint(LCRZOFILE *pf);
int lcrzo_priv_conf_devicenumbername_debug_fprint(LCRZOFILE *pf)
{ lcrzo_priv_confvars *ptrcv;
  lcrzo_priv_conf_devicenumbername dnn;
  lcrzo_int32 count, i;

  fprintf(pf, "\n\nDisplay each entry in list_devicenumbername\n");
 
  lcrzo_er(lcrzo_priv_confvars_ptr_get(&ptrcv));
  lcrzo_er(lcrzo_list_count(&ptrcv->list_devicenumbername, &count));
  for ( i=1 ; i<=count ; i++ )
  { lcrzo_er(lcrzo_list_value_pos(&ptrcv->list_devicenumbername, i, &dnn));
    fprintf(pf, "%lu\t", dnn.number);
    lcrzo_er(lcrzo_device_fprint(pf, "device=", dnn.name, "\n"));
  }

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_conf_devicenumbername_devices_add_system(lcrzo_uint32 devicenumber,
                                                        lcrzo_int16 hwtype,
	  		                                                lcrzo_constetha etha,
			                                                  lcrzo_ipl ipl,
			                                                  lcrzo_ipl netmask,
			                                                  lcrzo_uint32 mtu,
                                                        lcrzo_int16 addressnumber);
int lcrzo_priv_conf_devicenumbername_devices_add_system(lcrzo_uint32 devicenumber,
                                                        lcrzo_int16 hwtype,
	  		                                                lcrzo_constetha etha,
			                                                  lcrzo_ipl ipl,
			                                                  lcrzo_ipl netmask,
			                                                  lcrzo_uint32 mtu,
                                                        lcrzo_int16 addressnumber)
{ lcrzo_device device;
  lcrzo_bool isanalias;
  char *pc;
  int ret;

  /*
  This function is called in iphlpapi and snmp, which sort devices
  using their IP address. So, the alias setting is incorrect and has 
  to be computed here.
  */

  /* start by this function and not lcrzo_priv_conf_devicenumbername_item_get because
     the list cannot store aliases (ex: eth0 and eth0:1 will both have the number 3223) */
  ret=lcrzo_priv_conf_device_init_ipl(ipl, device);
  if (ret==LCRZO_ERR_OK)
  { /* set device */
    lcrzo_er(lcrzo_priv_conf_devicenumbername_item_add(devicenumber, device));
    /* guess if it's an alias */
    pc=strchr(device, ':');
    if (pc==NULL)
      isanalias=LCRZO_FALSE;
    else
      isanalias=LCRZO_TRUE;
  }
  else
  { ret=lcrzo_priv_conf_devicenumbername_item_get(devicenumber, device);
    if (ret!=LCRZO_ERR_OK)
    { ret=lcrzo_priv_conf_incomplete_add_1(devicenumber, hwtype, etha, ipl, netmask, mtu);
      return(ret);
      /* stop here */
    }
    /* use addressnumber to guess if it's an alias */
    if (addressnumber==1)
    { isanalias=LCRZO_FALSE;
    }
    else
    { isanalias=LCRZO_TRUE;
      /* add a fake alias number at the end of the device */
      lcrzo_er(lcrzo_string_append_text(":", LCRZO_DEVICE_MAXBYTES, device));
      lcrzo_er(lcrzo_string_append_uint32(addressnumber-1, "%u", 
					  LCRZO_DEVICE_MAXBYTES, device));
    }
  }

  /* check if it's possible to update the hardware type */
  ret=lcrzo_priv_hwtype_completewithdevice(hwtype, device, LCRZO_FALSE, &hwtype);
  if (ret!=LCRZO_ERR_OK) hwtype=LCRZO_DEVICE_HWTYPE_UNKNOWN;

  /* add it */
  ret=lcrzo_priv_conf_winras_devices_add_system(device, hwtype, etha, ipl, netmask, mtu,
                                         LCRZO_TRUE, isanalias);

  return(ret);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_conf_devicenumbername_arp_add_system(lcrzo_uint32 devicenumber,
						    lcrzo_constetha etha,
						    lcrzo_ipl ipl,
						    lcrzo_bool ispermanent);
int lcrzo_priv_conf_devicenumbername_arp_add_system(lcrzo_uint32 devicenumber,
						    lcrzo_constetha etha,
						    lcrzo_ipl ipl,
						    lcrzo_bool ispermanent)
{ lcrzo_device device;
  int ret;
  
  ret=lcrzo_priv_conf_devicenumbername_item_get(devicenumber, device);
  if (ret==LCRZO_ERR_OK)
  { ret=lcrzo_priv_conf_arp_add_system(device, etha, ipl, ispermanent);
  }

  return(ret);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_conf_devicenumbername_routes_add_system(lcrzo_uint32 devicenumber,
						       lcrzo_ipl dest,
						       lcrzo_ipl mask,
						       lcrzo_ipl ipsource,
						       lcrzo_ipl gateway,
						       lcrzo_uint32 metric,
						       lcrzo_bool isup);
int lcrzo_priv_conf_devicenumbername_routes_add_system(lcrzo_uint32 devicenumber,
						       lcrzo_ipl dest,
						       lcrzo_ipl mask,
						       lcrzo_ipl ipsource,
						       lcrzo_ipl gateway,
						       lcrzo_uint32 metric,
						       lcrzo_bool isup)
{ lcrzo_device device;
  int ret;
  
  ret=lcrzo_priv_conf_devicenumbername_item_get(devicenumber, device);
  if (ret==LCRZO_ERR_OK)
  { ret=lcrzo_priv_conf_routes_add_system(device, dest, mask, ipsource, gateway, metric, isup);
  }

  return(ret);
}

#else
 #error "Unknown value for LCRZODEF_SYSTEM"
#endif
