/* This file is part of Lcrzo.
*/

#ifdef LCRZODEF_SYSTEM_Unix
  /* this function is not needed*/

#elif defined LCRZODEF_SYSTEM_Windows

/*-------------------------------------------------------------*/
/* This critical section is for PacketOpenAdapter
   (PacketGetAdapterNames and pcap_open_live internally call it).
   We also use it for close functions. */
CRITICAL_SECTION lcrzo_priv_criticalsection_packetdll;

/*-------------------------------------------------------------*/
int lcrzo_priv_criticalsection_init(void)
{
  InitializeCriticalSection(&lcrzo_priv_criticalsection_packetdll);
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_criticalsection_close(void)
{
  DeleteCriticalSection(&lcrzo_priv_criticalsection_packetdll);
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
/* This function read registry and retrieve adapters.
   It is needed when lcrzo_priv_conf_winras_w95bugged returns true.
   The code is based on WinPcap 2.2, but doesn't open the adapter.
 */
int lcrzo_priv_w9x_PacketGetAdapterNames(PTSTR pStr, PULONG BufferSize);
int lcrzo_priv_w9x_PacketGetAdapterNames(PTSTR pStr, PULONG BufferSize)
{ LONG i, Result, Status;
	char *TpStr;
  HKEY Key, Key1;
	ULONG BSize, dim;
	char NdisName[32];

  Status=RegOpenKeyEx(HKEY_LOCAL_MACHINE,"SYSTEM",0,KEY_READ,&Key);
  if ( Status != ERROR_SUCCESS ) return(LCRZO_ERR_FUREGOPENKEYEX);
  Status=RegOpenKeyEx(Key,"CurrentControlSet",0,KEY_READ,&Key);
  if ( Status != ERROR_SUCCESS ) return(LCRZO_ERR_FUREGOPENKEYEX);
  Status=RegOpenKeyEx(Key,"Services",0,KEY_READ,&Key);
  if ( Status != ERROR_SUCCESS ) return(LCRZO_ERR_FUREGOPENKEYEX);
  Status=RegOpenKeyEx(Key,"class",0,KEY_READ,&Key);
  if ( Status != ERROR_SUCCESS ) return(LCRZO_ERR_FUREGOPENKEYEX);
  Status=RegOpenKeyEx(Key,"net",0,KEY_READ,&Key);
  if ( Status != ERROR_SUCCESS ) return(LCRZO_ERR_FUREGOPENKEYEX);

	TpStr=pStr;
	BSize=*BufferSize;
	i=0;
  Result=ERROR_SUCCESS;
	while(LCRZO_TRUE)
  { Result=RegEnumKey(Key,i,NdisName,32);
    if (Result!=ERROR_SUCCESS) break;
    i++;
    Status=RegOpenKeyEx(Key,NdisName,0,KEY_READ,&Key1);
	  if(Status!=ERROR_SUCCESS) continue;
	  Status=RegOpenKeyEx(Key1,"NDIS",0,KEY_READ,&Key1);
		if(Status!=ERROR_SUCCESS) continue;
		dim=BSize;
    Status=RegQueryValueEx(Key1,"LOGDRIVERNAME",NULL,NULL,(LPBYTE)TpStr,&dim);
		if(Status!=ERROR_SUCCESS) continue;
		BSize-=dim;
		TpStr+=dim;
	}
	
  if ( Result != ERROR_NO_MORE_ITEMS ) return(LCRZO_ERR_FUREGENUMKEY);

	TpStr[0]='\0';
	*BufferSize = *BufferSize - BSize + 1;

  /*lcrzo_data_print((lcrzo_data)pStr, *BufferSize, LCRZO_PRINTTYPE_DUMP);*/

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_cs_PacketGetAdapterNames(PTSTR pStr, 
                                        PULONG BufferSize)
{ BOOLEAN err;
  lcrzo_bool bugged;
  int ret;

  /* special case of RAS under Win95 : does not work */
  ret=lcrzo_priv_conf_winras_w95bugged(&bugged);
  if ( ret==LCRZO_ERR_OK && bugged )
  { ret=lcrzo_priv_w9x_PacketGetAdapterNames(pStr, BufferSize);
    if (ret!=LCRZO_ERR_OK)
    { /* set an empty list */
      if (pStr!=NULL)
      { pStr[0]=0;
	      pStr[1]=0;
      }
	    if (BufferSize!=NULL) *BufferSize=2;
    }
    return(LCRZO_ERR_OK);
  }

  EnterCriticalSection(&lcrzo_priv_criticalsection_packetdll);
  err=PacketGetAdapterNames(pStr, BufferSize);
  LeaveCriticalSection(&lcrzo_priv_criticalsection_packetdll);

  if (!err) return(LCRZO_ERR_FUPACKETGETADAPTERNAMES);
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_cs_PacketOpenAdapter(LPTSTR AdapterName,
                                    LPADAPTER *plp)
{ 
  EnterCriticalSection(&lcrzo_priv_criticalsection_packetdll);
  *plp=PacketOpenAdapter(AdapterName);
  LeaveCriticalSection(&lcrzo_priv_criticalsection_packetdll);

  if (*plp==NULL) return(LCRZO_ERR_FUPACKETOPENADAPTER);
  lcrzo_er(lcrzo_priv_err_purge_int());

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_cs_pcap_open_live(char *dev, 
                                 int mtu, 
                                 int promisc, 
                                 int toms, 
                                 char *err,
                                 pcap_t* *ppc)
{ 
  EnterCriticalSection(&lcrzo_priv_criticalsection_packetdll);
  *ppc=pcap_open_live(dev, mtu, promisc, toms, err);
  LeaveCriticalSection(&lcrzo_priv_criticalsection_packetdll);

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_cs_PacketCloseAdapter(LPADAPTER lpAdapter)
{ lcrzo_bool bugged;
  int ret;

  /* special case of RAS under Win95 : does not work */
  ret=lcrzo_priv_conf_winras_w95bugged(&bugged);
  if ( ret==LCRZO_ERR_OK && bugged )
  { return(LCRZO_ERR_OK);
  }
	
  EnterCriticalSection(&lcrzo_priv_criticalsection_packetdll);
  PacketCloseAdapter(lpAdapter);
  LeaveCriticalSection(&lcrzo_priv_criticalsection_packetdll);
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_cs_pcap_close(pcap_t *pt)
{ lcrzo_bool bugged;
  int ret;

  /* special case of RAS under Win95 : does not work */
  ret=lcrzo_priv_conf_winras_w95bugged(&bugged);
  if ( ret==LCRZO_ERR_OK && bugged )
  { return(LCRZO_ERR_OK);
  }

  EnterCriticalSection(&lcrzo_priv_criticalsection_packetdll);
  pcap_close(pt);
  LeaveCriticalSection(&lcrzo_priv_criticalsection_packetdll);
  return(LCRZO_ERR_OK);
}

#else
 #error "Unknown value for LCRZODEF_SYSTEM"
#endif
