/* This file is part of Lcrzo.
*/

#if defined LCRZODEF_SYSTEM_Unix
 /* do no need these functions */
#elif defined LCRZODEF_SYSTEM_Windows

/*-------------------------------------------------------------*/
lcrzo_priv_dll_snmp lcrzo_global_dll_snmp;

/*-------------------------------------------------------------*/
int lcrzo_priv_dll_snmpoid_alloc(lcrzo_int32 idLength,
                                 AsnObjectIdentifier *ObjId);
int lcrzo_priv_dll_snmpoid_alloc(lcrzo_int32 idLength,
                                 AsnObjectIdentifier *ObjId)
{ 
  ObjId->idLength=idLength;
  /* we cannot use lcrzo_data_alloc because lcrzo_priv_dll_snmpoid_free
     cannot use lcrzo_data_free2 */
  ObjId->ids = (UINT *)GlobalAlloc(GMEM_ZEROINIT, idLength*sizeof(UINT));
  if (ObjId->ids==NULL) return(LCRZO_ERR_FUGLOBALALLOC);
  
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_dll_snmpoid_free(AsnObjectIdentifier *ObjId);
int lcrzo_priv_dll_snmpoid_free(AsnObjectIdentifier *ObjId)
{ void *pret;

  /* we cannot use lcrzo_data_free2 because this function is sometimes used
     to free items which are allocated by SNMP DLL with a GlobalAlloc */
  pret=GlobalFree(ObjId->ids);
  if (pret!=NULL) return(LCRZO_ERR_FUGLOBALFREE);
  ObjId->ids = NULL;
  ObjId->idLength = 0;
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_dll_snmpoid_cpy(AsnObjectIdentifier *DstObjId,
                               AsnObjectIdentifier *SrcObjId);
int lcrzo_priv_dll_snmpoid_cpy(AsnObjectIdentifier *DstObjId,
                               AsnObjectIdentifier *SrcObjId)
{ 
  lcrzo_er(lcrzo_priv_dll_snmpoid_alloc(SrcObjId->idLength, DstObjId));
  memcpy(DstObjId->ids, SrcObjId->ids, SrcObjId->idLength*sizeof(UINT));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_dll_snmpoid_init(AsnObjectIdentifier *DstObjId, ...)
{ int count, i;
  va_list marker;

  /* count the number of arguments */
  va_start(marker, DstObjId);
  count=0;
  i=va_arg(marker, int);
  while( i != -1 )
  { count++;
    i=va_arg(marker, int);
  }
  va_end(marker);  
  
  /* initialize memory */
  lcrzo_er(lcrzo_priv_dll_snmpoid_alloc(count, DstObjId));

  /* set values */
  va_start(marker, DstObjId);
  for ( i=0 ; i<count ; i++ )
  { DstObjId->ids[i]=va_arg(marker, int);
  }
  va_end(marker);  

  return LCRZO_ERR_OK;
}

/*-------------------------------------------------------------*/
int lcrzo_priv_dll_snmpoid_close(AsnObjectIdentifier *ObjId)
{ 
  lcrzo_er(lcrzo_priv_dll_snmpoid_free(ObjId));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_dll_snmpoid_print(AsnObjectIdentifier *ObjId)
{ unsigned int i;

  for ( i=0 ; i<ObjId->idLength-1 ; i++ )
  { printf("%d.", ObjId->ids[i]);
  }
  printf("%d", ObjId->ids[ObjId->idLength-1]);

  return(LCRZO_ERR_OK);
}


/*-------------------------------------------------------------*/
int lcrzo_SnmpUtilVarBindFree(IN OUT SnmpVarBind *VarBind);
int lcrzo_SnmpUtilVarBindFree(IN OUT SnmpVarBind *VarBind)
{ BYTE asnType;
  void *pret;

  /* free object name */
  lcrzo_er(lcrzo_priv_dll_snmpoid_free(&VarBind->name));

  asnType = VarBind->value.asnType;
  if(asnType==ASN_OBJECTIDENTIFIER)
  { lcrzo_er(lcrzo_priv_dll_snmpoid_free(&VarBind->value.asnValue.object));
  }
  else if(
        (asnType==ASN_OCTETSTRING) ||
        (asnType==ASN_IPADDRESS) ||
        (asnType==ASN_OPAQUE) ||
        (asnType==ASN_SEQUENCE))
  { if(VarBind->value.asnValue.string.dynamic)
    { /* this variable is not allocated by lcrzo, so we cannot use
         lcrzo_data_free2 */
      pret=GlobalFree(VarBind->value.asnValue.string.stream);
      if (pret!=NULL) return(LCRZO_ERR_FUGLOBALFREE);
    }
  }
  VarBind->value.asnType = ASN_NULL;

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
typedef BOOL(WINAPI * pSnmpExtensionInit) (
        IN DWORD dwTimeZeroReference,
        OUT HANDLE * hPollForTrapEvent,
        OUT AsnObjectIdentifier * supportedView);

/*-------------------------------------------------------------*/
int lcrzo_priv_dll_snmp_init(lcrzo_priv_dll_snmp *pdll_snmp)
{ pSnmpExtensionInit snmpe_init;
  HANDLE PollForTrapEvent;
  AsnObjectIdentifier SupportedView;
  int ret;

  pdll_snmp->isloaded=LCRZO_FALSE;

  /* load the DLL */
  pdll_snmp->hinstance=LoadLibrary("inetmib1.dll");
  if ( pdll_snmp->hinstance==NULL )
  { return(LCRZO_ERR_FULOADLIBRARY);
  }

  /* init query function */
  pdll_snmp->snmpqueryfunction=(pSnmpExtensionQuery)GetProcAddress(pdll_snmp->hinstance, "SnmpExtensionQuery");
  if ( pdll_snmp->snmpqueryfunction==NULL )
  { FreeLibrary(pdll_snmp->hinstance);
    return(LCRZO_ERR_FUGETPROCADDRESS);
  }

  /* initialize the extension agent */
  snmpe_init=(pSnmpExtensionInit)GetProcAddress(pdll_snmp->hinstance, "SnmpExtensionInit");
  if ( snmpe_init==NULL )
  { FreeLibrary(pdll_snmp->hinstance);
    return(LCRZO_ERR_FUGETPROCADDRESS);
  }
  ret=snmpe_init(GetTickCount(), &PollForTrapEvent, &SupportedView);
  if (!ret)
  { FreeLibrary(pdll_snmp->hinstance);
    return(LCRZO_ERR_FUSNMPEXTENSIONINIT);
  }
  /*lcrzo_priv_AsnOid_print(&SupportedView); puts("");*/
  /* PollForTrapEvent and SupportedView are automatically closed by the dll */

  pdll_snmp->isloaded=LCRZO_TRUE;
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_dll_snmp_close(lcrzo_priv_dll_snmp *pdll_snmp)
{ int ret;

  if (!pdll_snmp->isloaded) return(LCRZO_ERR_OK);

  /* unload the DLL */
  ret=FreeLibrary(pdll_snmp->hinstance);
  if (!ret) return(LCRZO_ERR_FUFREELIBRARY);

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_dll_snmpreq_init(lcrzo_priv_dll_snmpreq *pdll_snmpreq,
                               AsnObjectIdentifier *poid1,
                               AsnObjectIdentifier *poid2,
                               AsnObjectIdentifier *poid3,
                               AsnObjectIdentifier *poid4,
                               AsnObjectIdentifier *poid5)
{ 
  if (!lcrzo_global_dll_snmp.isloaded) return(LCRZO_ERR_IEUNIMPLEMENTED);
 
  /* varBind will be used as the array */
  pdll_snmpreq->varBindList.list = pdll_snmpreq->varBind;

  /* if there is no oid */
  if ( poid1==NULL )
  { pdll_snmpreq->varBindList.len=0;
    return(LCRZO_ERR_OK);
  }
  
  /* first oid is set */
  pdll_snmpreq->varBindList.len=1;
  lcrzo_er(lcrzo_priv_dll_snmpoid_cpy(&(pdll_snmpreq->varBindList.list[0].name), poid1));
  pdll_snmpreq->varBindList.list[0].value.asnType=ASN_NULL;
  
  /* second oid is set */
  if ( poid2!=NULL )
  { pdll_snmpreq->varBindList.len=2;
    pdll_snmpreq->varBindList.list[1].value.asnType=ASN_NULL;
    lcrzo_er(lcrzo_priv_dll_snmpoid_cpy(&(pdll_snmpreq->varBindList.list[1].name), poid2));
  }

  /* third oid is set */
  if ( poid3!=NULL )
  { pdll_snmpreq->varBindList.len=3;
    pdll_snmpreq->varBindList.list[2].value.asnType=ASN_NULL;
    lcrzo_er(lcrzo_priv_dll_snmpoid_cpy(&(pdll_snmpreq->varBindList.list[2].name), poid3));
  }

  /* fourth oid is set */
  if ( poid4!=NULL )
  { pdll_snmpreq->varBindList.len=4;
    pdll_snmpreq->varBindList.list[3].value.asnType=ASN_NULL;
    lcrzo_er(lcrzo_priv_dll_snmpoid_cpy(&(pdll_snmpreq->varBindList.list[3].name), poid4));
  }

  /* fifth oid is set */
  if ( poid5!=NULL )
  { pdll_snmpreq->varBindList.len=5;
    pdll_snmpreq->varBindList.list[4].value.asnType=ASN_NULL;
    lcrzo_er(lcrzo_priv_dll_snmpoid_cpy(&(pdll_snmpreq->varBindList.list[4].name), poid5));
  }

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_dll_snmpreq_close(lcrzo_priv_dll_snmpreq *pdll_snmpreq)
{ unsigned int i;

  for ( i=0 ; i<pdll_snmpreq->varBindList.len ; i++ )
  { lcrzo_er(lcrzo_SnmpUtilVarBindFree(&(pdll_snmpreq->varBindList.list[i])));
  }
  pdll_snmpreq->varBindList.len=0;

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_dll_snmpreq_print(lcrzo_priv_dll_snmpreq *pdll_snmpreq)
{ unsigned int i, j;

  printf("varBindList : %d\n", pdll_snmpreq->varBindList.len);
  for ( i=0 ; i<pdll_snmpreq->varBindList.len ; i++ )
  { /* display name */
    lcrzo_er(lcrzo_priv_dll_snmpoid_print(&(pdll_snmpreq->varBindList.list[i].name)));
    /* display value type */
    printf(" Type %d(", pdll_snmpreq->varBindList.list[i].value.asnType);
    switch(pdll_snmpreq->varBindList.list[i].value.asnType)
    { case ASN_INTEGER: printf("ASN_INTEGER"); break;
      case ASN_OCTETSTRING: printf("ASN_OCTETSTRING"); break;
      case ASN_NULL: printf("ASN_NULL"); break;
      case ASN_OBJECTIDENTIFIER: printf("ASN_OBJECTIDENTIFIER"); break;
      case ASN_SEQUENCE: printf("ASN_SEQUENCE"); break;
      case ASN_IPADDRESS: printf("ASN_IPADDRESS"); break;
      case ASN_COUNTER32: printf("ASN_COUNTER32"); break;
      case ASN_GAUGE32: printf("ASN_GAUGE32"); break;
      case ASN_TIMETICKS: printf("ASN_TIMETICKS"); break;
      case ASN_OPAQUE: printf("ASN_OPAQUE"); break;
      default: printf("???"); break;
    }
    printf(")\n");
    /* display the value */
    switch(pdll_snmpreq->varBindList.list[i].value.asnType)
    { case ASN_INTEGER:
        printf(" %lu\n", (unsigned long)pdll_snmpreq->varBindList.list[i].value.asnValue.number);
        break;
      case ASN_OCTETSTRING:
        printf("len=%d\n", pdll_snmpreq->varBindList.list[i].value.asnValue.string.length);
        lcrzo_er(lcrzo_data_print((lcrzo_data)pdll_snmpreq->varBindList.list[i].value.asnValue.string.stream,
                         (lcrzo_int32)pdll_snmpreq->varBindList.list[i].value.asnValue.string.length, LCRZO_PRINTTYPE_DUMP));
        break;
      case ASN_NULL:
        /* nothing to print */
        break;
      case ASN_COUNTER32:
        printf(" %lu\n", pdll_snmpreq->varBindList.list[i].value.asnValue.counter);
        break;
      case ASN_GAUGE32:
        printf(" %lu\n", pdll_snmpreq->varBindList.list[i].value.asnValue.gauge);
        break;
      case ASN_TIMETICKS:
        printf(" %lu\n", pdll_snmpreq->varBindList.list[i].value.asnValue.ticks);
        break;
      case ASN_OBJECTIDENTIFIER:
        /* display name */
        printf(" ");
        lcrzo_er(lcrzo_priv_dll_snmpoid_print(&(pdll_snmpreq->varBindList.list[i].value.asnValue.object)));
        printf("\n");
        break;
      case ASN_IPADDRESS:
        printf(" ");
        for ( j=0 ; j<pdll_snmpreq->varBindList.list[i].value.asnValue.address.length ; j++ )
        { printf("%d.", pdll_snmpreq->varBindList.list[i].value.asnValue.address.stream[j]); 
        }
        printf("\n");
        break;
      case ASN_SEQUENCE:
      case ASN_OPAQUE:
      default:
        return(LCRZO_ERR_IEUNIMPLEMENTED);
    }
  }

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_dll_snmpreq__uint32(lcrzo_priv_dll_snmpreq *pdll_snmpreq,
                                  lcrzo_uint8 index,
                                  lcrzo_uint32 *pnumber)
{ lcrzo_uint32 number;

  if ( index >= pdll_snmpreq->varBindList.len ) return(LCRZO_ERR_POTOOBIG);

  switch(pdll_snmpreq->varBindList.list[index].value.asnType)
  { case ASN_INTEGER:
      number=pdll_snmpreq->varBindList.list[index].value.asnValue.number;
      break;
    case ASN_COUNTER32:
      number=pdll_snmpreq->varBindList.list[index].value.asnValue.counter;
      break;
    case ASN_GAUGE32:
      number=pdll_snmpreq->varBindList.list[index].value.asnValue.gauge;
      break;
    case ASN_TIMETICKS:
      number=pdll_snmpreq->varBindList.list[index].value.asnValue.ticks;
      break;
    case ASN_IPADDRESS:
      if (pdll_snmpreq->varBindList.list[index].value.asnValue.address.length!=4)
      { number=0;
      }
      else
      { number = (pdll_snmpreq->varBindList.list[index].value.asnValue.address.stream[0]<<24) |
                 (pdll_snmpreq->varBindList.list[index].value.asnValue.address.stream[1]<<16) |
                 (pdll_snmpreq->varBindList.list[index].value.asnValue.address.stream[2]<<8 ) |
                 (pdll_snmpreq->varBindList.list[index].value.asnValue.address.stream[3]    ) ;
      }
      break;
    case ASN_OCTETSTRING:
    case ASN_NULL:
    case ASN_OBJECTIDENTIFIER:
    case ASN_SEQUENCE:
    case ASN_OPAQUE:
    default:
      return(LCRZO_ERR_SPSNMPDLLBADTYPE);
  }

  if ( pnumber!=NULL ) *pnumber=number;

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_dll_snmpreq__data(lcrzo_priv_dll_snmpreq *pdll_snmpreq,
                                lcrzo_uint8 index,
                                lcrzo_data *pdata,
                                lcrzo_int32 *pdatasize)
{ 
  if ( index >= pdll_snmpreq->varBindList.len ) return(LCRZO_ERR_POTOOBIG);

  switch(pdll_snmpreq->varBindList.list[index].value.asnType)
  { case ASN_OCTETSTRING:
      lcrzo_er(lcrzo_data_initm_data((lcrzo_data)pdll_snmpreq->varBindList.list[index].value.asnValue.string.stream,
                                     pdll_snmpreq->varBindList.list[index].value.asnValue.string.length,
                                     pdata, pdatasize));
      break;
    case ASN_IPADDRESS:
      lcrzo_er(lcrzo_data_initm_data((lcrzo_data)pdll_snmpreq->varBindList.list[index].value.asnValue.address.stream,
                                     pdll_snmpreq->varBindList.list[index].value.asnValue.address.length,
                                     pdata, pdatasize));
      break;
    case ASN_NULL:
    case ASN_INTEGER:
    case ASN_SEQUENCE:
    case ASN_OPAQUE:
    case ASN_COUNTER32:
    case ASN_GAUGE32:
    case ASN_TIMETICKS:
    default:
      return(LCRZO_ERR_SPSNMPDLLBADTYPE);
  }

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_dll_snmpreq__etha(lcrzo_priv_dll_snmpreq *pdll_snmpreq,
                                lcrzo_uint8 index,
                                lcrzo_etha etha)
{ 
  if ( index >= pdll_snmpreq->varBindList.len ) return(LCRZO_ERR_POTOOBIG);

  switch(pdll_snmpreq->varBindList.list[index].value.asnType)
  { case ASN_OCTETSTRING:
      if (pdll_snmpreq->varBindList.list[index].value.asnValue.string.length==LCRZO_ETHA_MAXBYTES)
      { lcrzo_er(lcrzo_etha_init(pdll_snmpreq->varBindList.list[index].value.asnValue.string.stream[0], 
                                 pdll_snmpreq->varBindList.list[index].value.asnValue.string.stream[1], 
                                 pdll_snmpreq->varBindList.list[index].value.asnValue.string.stream[2], 
                                 pdll_snmpreq->varBindList.list[index].value.asnValue.string.stream[3], 
                                 pdll_snmpreq->varBindList.list[index].value.asnValue.string.stream[4], 
                                 pdll_snmpreq->varBindList.list[index].value.asnValue.string.stream[5], 
                                 etha));
      }
      else
      { lcrzo_er(lcrzo_etha_initdefault(etha));
      }
      break;
    case ASN_IPADDRESS:
    case ASN_NULL:
    case ASN_INTEGER:
    case ASN_SEQUENCE:
    case ASN_OPAQUE:
    case ASN_COUNTER32:
    case ASN_GAUGE32:
    case ASN_TIMETICKS:
    default:
      return(LCRZO_ERR_SPSNMPDLLBADTYPE);
  }

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_dll_snmpreq_getnext(lcrzo_priv_dll_snmpreq *pdll_snmpreq)
{ AsnInteger32 errorStatus;
  AsnInteger32 errorIndex;
  int ret;

  if (!lcrzo_global_dll_snmp.isloaded) return(LCRZO_ERR_IEUNIMPLEMENTED);
 
  ret = (lcrzo_global_dll_snmp.snmpqueryfunction)(SNMP_PDU_GETNEXT, &(pdll_snmpreq->varBindList), &errorStatus, &errorIndex);
  if ( !ret ) return(LCRZO_ERR_FUSNMPEXTENSIONQUERY);
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_dll_snmp_printall()
{ lcrzo_priv_dll_snmpreq dll_snmpreq;
  AsnObjectIdentifier oid0;
  
  if (!lcrzo_global_dll_snmp.isloaded) return(LCRZO_ERR_IEUNIMPLEMENTED);

  lcrzo_er(lcrzo_priv_dll_snmpoid_init(&oid0, 1, 3, 6, 1, 2, 1, -1));
  lcrzo_er(lcrzo_priv_dll_snmpreq_init(&dll_snmpreq, &oid0, NULL, NULL, NULL, NULL));
  lcrzo_er(lcrzo_priv_dll_snmpoid_close(&oid0));
  while ( dll_snmpreq.varBindList.list[0].name.idLength<7 || dll_snmpreq.varBindList.list[0].name.ids[6] < 10 )
  { lcrzo_er(lcrzo_priv_dll_snmpreq_getnext(&dll_snmpreq));
    lcrzo_er(lcrzo_priv_dll_snmpreq_print(&dll_snmpreq));
  }
  lcrzo_er(lcrzo_priv_dll_snmpreq_close(&dll_snmpreq));

  return(LCRZO_ERR_OK);
}

#else
 #error "Unknown value for LCRZODEF_SYSTEM"
#endif
