/* This file is part of Lcrzo.
*/

#if defined LCRZODEF_SYSTEM_Unix
 /* do no need these functions */
#elif defined LCRZODEF_SYSTEM_Windows

/*-------------------------------------------------------------*/
lcrzo_priv_dll_winsock2 lcrzo_global_dll_winsock2;

/*-------------------------------------------------------------*/
int lcrzo_priv_dll_winsock2_needtoload(lcrzo_bool *pyes)
{ lcrzo_priv_globalvars *ptr;
  lcrzo_bool b;

  /* Currently, we only use winsock 2.2 to spoof at IP level, which
     works on win2k and winxp */

  lcrzo_er(lcrzo_priv_globalvars_ptr_get(&ptr));
  switch((int)ptr->windows_systemversion)
  { case LCRZO_WINDOWS_SYSTEMVERSION_95:
    case LCRZO_WINDOWS_SYSTEMVERSION_98:
    case LCRZO_WINDOWS_SYSTEMVERSION_me:
    case LCRZO_WINDOWS_SYSTEMVERSION_nt4:
      b=LCRZO_FALSE;
      break;
    case LCRZO_WINDOWS_SYSTEMVERSION_2k:
    case LCRZO_WINDOWS_SYSTEMVERSION_xp:
    default:
      b=LCRZO_TRUE;
      break;
  }

  if (pyes!=NULL) *pyes=b;
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_dll_winsock2_init(lcrzo_priv_dll_winsock2 *pdll_winsock2)
{ WSADATA wsaData;
  int major;
  int minor;
  int ret; 
 
  pdll_winsock2->isloaded=LCRZO_FALSE;

  /* load the DLL */
  pdll_winsock2->hinstance=LoadLibrary("ws2_32.dll");
  if ( pdll_winsock2->hinstance==NULL )
  { return(LCRZO_ERR_FULOADLIBRARY);
  }

  /* init functions */
  pdll_winsock2->WSAStartup=(pWSAStartup)GetProcAddress(pdll_winsock2->hinstance, "WSAStartup");
  if ( pdll_winsock2->WSAStartup==NULL )
  { FreeLibrary(pdll_winsock2->hinstance);
    return(LCRZO_ERR_FUGETPROCADDRESS);
  }
  pdll_winsock2->WSACleanup=(pWSACleanup)GetProcAddress(pdll_winsock2->hinstance, "WSACleanup");
  if ( pdll_winsock2->WSACleanup==NULL )
  { FreeLibrary(pdll_winsock2->hinstance);
    return(LCRZO_ERR_FUGETPROCADDRESS);
  }
  pdll_winsock2->socket=(psocket)GetProcAddress(pdll_winsock2->hinstance, "socket");
  if ( pdll_winsock2->socket==NULL )
  { FreeLibrary(pdll_winsock2->hinstance);
    return(LCRZO_ERR_FUGETPROCADDRESS);
  }
  pdll_winsock2->closesocket=(pclosesocket)GetProcAddress(pdll_winsock2->hinstance, "closesocket");
  if ( pdll_winsock2->closesocket==NULL )
  { FreeLibrary(pdll_winsock2->hinstance);
    return(LCRZO_ERR_FUGETPROCADDRESS);
  }
  pdll_winsock2->setsockopt=(psetsockopt)GetProcAddress(pdll_winsock2->hinstance, "setsockopt");
  if ( pdll_winsock2->setsockopt==NULL )
  { FreeLibrary(pdll_winsock2->hinstance);
    return(LCRZO_ERR_FUGETPROCADDRESS);
  }
  pdll_winsock2->sendto=(psendto)GetProcAddress(pdll_winsock2->hinstance, "sendto");
  if ( pdll_winsock2->sendto==NULL )
  { FreeLibrary(pdll_winsock2->hinstance);
    return(LCRZO_ERR_FUGETPROCADDRESS);
  }

  /* init the wanted version */
  major=2; minor=2;
  ret = pdll_winsock2->WSAStartup(MAKEWORD(major, minor), &wsaData); 
  if (ret != 0)
  { FreeLibrary(pdll_winsock2->hinstance);
    return(LCRZO_ERR_FUWSASTARTUP);
  }
 
  if ( HIBYTE( wsaData.wVersion ) != major || 
       LOBYTE( wsaData.wVersion ) != minor ) 
  { pdll_winsock2->WSACleanup();
    FreeLibrary(pdll_winsock2->hinstance);
    return(LCRZO_ERR_FUWSASTARTUPVER);
  }

  pdll_winsock2->isloaded=LCRZO_TRUE;
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_dll_winsock2_close(lcrzo_priv_dll_winsock2 *pdll_winsock2)
{ int ret;

  if (!pdll_winsock2->isloaded) return(LCRZO_ERR_OK);

  /* unload the DLL */
  ret=FreeLibrary(pdll_winsock2->hinstance);
  if (!ret) return(LCRZO_ERR_FUFREELIBRARY);

  return(LCRZO_ERR_OK);
}

#else
 #error "Unknown value for LCRZODEF_SYSTEM"
#endif
