/* This file is part of Lcrzo.
*/

int lcrzo_priv_wait_winhandle_wait(lcrzo_winhandle *pwinhandle, 
                              lcrzo_time *pmaxwaitingtime, 
                              lcrzo_bool *pevent)
#if defined LCRZODEF_SYSTEM_Unix
{ return(LCRZO_ERR_IEUNIMPLEMENTED);
  pwinhandle=pwinhandle;
  pmaxwaitingtime=pmaxwaitingtime;
  pevent=pevent;
}
#elif defined LCRZODEF_SYSTEM_Windows
{ DWORD err, msec;
  lcrzo_bool event;
  DWORD nbevent;
  INPUT_RECORD ir[5];
  DWORD n, i;
  DWORD readbytes, bytesavail;

  event=LCRZO_FALSE;
  switch(pwinhandle->type)
  { 
    case LCRZO_PRIV_WAIT_SOURCE_WINHANDLE_FILE:
      /* it's a filehandle for a disk file, for which there is
         only data, or endoffile. So there is no need to wait. */
      event=LCRZO_TRUE;
      break;

    case LCRZO_PRIV_WAIT_SOURCE_WINHANDLE_UNKNOWN:
      /* wait during the specified time */
      lcrzo_er(lcrzo_priv_wait_winmsec_init_time(pmaxwaitingtime, &msec));
      err=WaitForSingleObject(pwinhandle->thehandle, msec);
      if (err==WAIT_FAILED) return(LCRZO_ERR_FUWAITFORSINGLEOBJECT);
      /* no data to read */
      if (err==WAIT_TIMEOUT) event=LCRZO_FALSE;
      else if (err==WAIT_OBJECT_0) event=LCRZO_TRUE;
      else return(LCRZO_ERR_FUWAITFORSINGLEOBJECT);
      break;

    case LCRZO_PRIV_WAIT_SOURCE_WINHANDLE_CONSOLE:
      /* wait during the specified time */
      lcrzo_er(lcrzo_priv_wait_winmsec_init_time(pmaxwaitingtime, &msec));
      err=WaitForSingleObject(pwinhandle->thehandle, msec);
      if (err==WAIT_FAILED) return(LCRZO_ERR_FUWAITFORSINGLEOBJECT);
      /* no data to read */
      if (err==WAIT_TIMEOUT) event=LCRZO_FALSE;
      else if (err==WAIT_OBJECT_0) event=LCRZO_TRUE;
      else return(LCRZO_ERR_FUWAITFORSINGLEOBJECT);
      if (event)
      { /* check there is really data */
        err=GetNumberOfConsoleInputEvents(pwinhandle->thehandle, &nbevent);
        if (err)
        { /*printf("nbevent=%d\n", nbevent);*/
          err=PeekConsoleInput(pwinhandle->thehandle, ir, 4, &n);
          if (!err) return(LCRZO_ERR_FUPEEKCONSOLEINPUT);
          event=LCRZO_FALSE;
          for ( i=0; i<n; i++)
          { /*printf("Type=%d\t", ir[i].EventType);
            printf("c=%c=%02x b=%d\n", ir[i].Event.KeyEvent.uChar.AsciiChar,
              ir[i].Event.KeyEvent.uChar.AsciiChar,
              ir[i].Event.KeyEvent.bKeyDown);*/
            if ( ir[i].EventType==KEY_EVENT && ir[i].Event.KeyEvent.bKeyDown )
            { event=LCRZO_TRUE;
            }
          }
        }
      }
      break;

    case LCRZO_PRIV_WAIT_SOURCE_WINHANDLE_PIPE:
      err=PeekNamedPipe(pwinhandle->thehandle, NULL, 0xFFFF, &readbytes, &bytesavail, NULL);
      if (!err)
      { /* end event */ 
        event=LCRZO_TRUE;
      }
      else
      { if ( readbytes || bytesavail ) event=LCRZO_TRUE;
      }
      break;

    default:
      return(lcrzo_ieuv("lcrzo_priv_wait_winhandle_wait", "pwinhandle->type"));
  }

  if (pevent!=NULL) *pevent=event;
  return(LCRZO_ERR_OK);
}
#else
 #error "Unknown value for LCRZODEF_SYSTEM"
#endif

